use Test::More;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Amazon::API::Object::PagerAWSItems;
use DE_EPAGES::Amazon::API::Object::RemoteAWS;

sub testPager {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
          SKIP: {
                my $Portal     = LoadObjectByPath('/Portals/Amazon');
                my $PortalSite = $Portal->path('Amazon_de_DE');

                my $RemoteAWS = DE_EPAGES::Amazon::API::Object::RemoteAWS->new(
                    'AWSURL'         => $PortalSite->get('AWSURL'),
                    'AWSAccessKeyId' => $Portal->get('AWSKeyID'),
                    'AWSSecretKeyId' => $Portal->get('AWSSecretKeyID'),
                    'Logging'        => $Portal->get('Logging')
                );

                my $Pager = DE_EPAGES::Amazon::API::Object::PagerAWSItems->new( {
                        'ObjectID'     => 1,
                        'RemoteAWS'    => $RemoteAWS,
                        'ASIN'         => undef,
                        'UPCEAN'       => undef,
                        'SearchString' => 'Deuter',
                        'ProductName'  => 'Deuter',
                        'Page'         => 1,
                    }
                );

                my $Page  = $Pager->page;
                my $Items = $Page->{Pager}->{Items};
                cmp_ok( scalar @{$Items}, '>', 0, ( scalar @{$Items} ) . ' results found for AWS keyword search.' );

                my $Pager2 = DE_EPAGES::Amazon::API::Object::PagerAWSItems->new( {
                        'ObjectID'  => 1,
                        'RemoteAWS' => $RemoteAWS,
                        'ASIN'      => 'B0009VC8MI',
                        'UPCEAN'    => undef,
                    }
                );
                if (
                    defined( $Pager2->{'XMLResponse'}->{'OperationRequest'}->{'Errors'}->{'Error'} )
                    && $Pager2->{'XMLResponse'}->{'OperationRequest'}->{'Errors'}->{'Error'}->[0]->{'Code'} eq 'AWS.TimestampExceedsTimeToLive'
                  )
                {
                    diag('AWS.TimestampExceedsTimeToLive - wrong clock! ASIN search will fail.');
                }
                my $Page2  = $Pager2->page;
                my $Items2 = $Page2->{Pager}->{Items};
                cmp_ok( scalar @{$Items2}, '==', 1, '1 pager result found for AWS ASIN search.' );
            }
        }
    );
}

sub Main {
    testPager();
    done_testing();
    return;
}

RunScript( Sub => \&Main );

1;
