use strict;
use Test::More;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/Scripts");

use Perl::Critic::Policy::DE_EPAGES::Variables::ProhibitPathConcatenation; ##no critic(RequireCorrectImports)
# Tests Perl::Critic::Policy::DE_EPAGES::Variables::ProhibitPathConcatenation
#

sub Main {

    my $critic = Perl::Critic->new(
        -theme    => 'epages',
        -severity => 5,
        -profile  => 'NONE',
    );
    ok( defined $critic, 'Perl::Critic initialized' );

    $critic->add_policy( -policy => 'DE_EPAGES::Variables::ProhibitPathConcatenation' );
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Variables::ProhibitPathConcatenation' }
    $critic->policies();
    ok( $policyLoaded, 'Policy loaded' );

    ##no critic qw(ProhibitImplicitNewlines ProhibitMissingImports ProhibitUnusedVariables RequireInterpolationOfMetachars)
    my $sourceCode = q(
        statement();
        my $alles_prima = 1;
    );
    ok( !$critic->critique( \$sourceCode ), 'funtion not found' );

    $sourceCode = q(
        LoadObjectByPath('/hallo');
    );
    ok( !$critic->critique( \$sourceCode ), 'function used, no concat' );

    $sourceCode = q(
        LoadObjectByPath($hallo);
    );
    ok( !$critic->critique( \$sourceCode ), 'function used, no concat' );

    $sourceCode = q(
        my $string1 = '/path';
        my $string2 = '/sub';
        LoadObjectByPath($string1 . $string2);
    );
    ok( $critic->critique( \$sourceCode ), 'function used, concat' );

    $sourceCode = q(
        my $string1 = '/path';
        my $string2 = '/sub';
        LoadObjectByPath("$string1 $string2");
    );
    ok( $critic->critique( \$sourceCode ), 'function used, variables in quotes' );

    $sourceCode = q(
        my $string1 = '/path';
        my $string2 = '/sub';
        ExistsObjectByPath($string1 . $string2);
    );
    ok( $critic->critique( \$sourceCode ), 'function used, concat' );

    $sourceCode = q(
        ExistsObjectByPath('/path /to');
    );
    ok( $critic->critique( \$sourceCode ), 'function used, spaces on quote 1' );

    $sourceCode = q(
        ExistsObjectByPath('/path /to');
    );
    ok( $critic->critique( \$sourceCode ), 'function used, spaces on quote 2' );

    $sourceCode = q(
        my $string1 = '/path';
        ExistsObjectByPath(" $string1");
    );
    ok( $critic->critique( \$sourceCode ), 'function used,  spaces on quote 3' );

    $sourceCode = q(
        my $string1 = '/path';
        my $string2 = '/sub';
        ExistsObjectByPath("$string1 $string2");
    );
    ok( $critic->critique( \$sourceCode ), 'function used, variables in quotes' );

    $sourceCode = q(
        my $string1 = '/path';
        LoadObjectByPath("/Shops/$string1");
    );
    ok( $critic->critique( \$sourceCode ), 'function used, variables in quote' );

    $sourceCode = q(
        my $string1 = '/path';
        my $string2 = '/sub';
        ExistsObjectByPath2("$string1 $string2");
    );
    ok( ! $critic->critique( \$sourceCode ), 'other function used 1' );

    $sourceCode = q(
        my $string1 = '/path';
        my $string2 = '/sub';
        _ExistsObjectByPath("$string1 $string2");
    );
    ok( ! $critic->critique( \$sourceCode ), 'other function used 2' );

    $sourceCode = q(
           return LoadObjectByPath('/ShippingFulfillmentTypes/'.$TransactionPaymentName);
    );
    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'mixed string and variable in return statement' ) or diag(@violations);

    done_testing();
}

RunScript( Sub => \&Main );
