/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.core
 *
 * @revision		$Revision: 1.7.2.5 $
 */

jQuery.plugin( 'ep.core', [
	'ep.modify',
	'jQuery.scope',
	'jQuery.each',
	'jQuery.mixin',
	'jQuery.event.fix'
], function( $, undefined ){

	// subclass of jQuery
	var ep = $.subclass();

	// merge config params
	ep.config = $.extend(
		{
			debugLevel:					0,
   			version:					'',
   			sessionId:					undefined,
   			webRoot:					'',
   			storeWebRoot:				'',
   			storeRoot:					'',
   			storeTypeRoot:				'',
   			siteRoot:					'',
   			siteId:						'0',
  			baseUrl:					undefined,
   			iconsRoot:					'',
   			objectId:					'0',
   			viewAction:					'',
   			storeFrontUrl:				'',
   			requestProtocolAndServer:	'',
   			maxContentLength:			'',
   			currency:					'',
   			autoMetaparse:				true
		},
		window.epConfig
	);

	var splice = Array.prototype.splice,
		push = Array.prototype.push,
		extend = $.extend,
		mixin = $.mixin,
		ready = $.fn.ready,
		browser = $.browser,
		browserClass = (/khtml/i).test(navigator.userAgent) ? 'khtml' : '';

	// add browser engine name as class to html tag
	$.each( ['mozilla', 'msie', 'webkit', 'opera'], function( i, name ){
		if( browser[name] ){
			browserClass = name;
			return false;
		}
	});
	$('html').addClass( browserClass );

	$.extend(ep,{
		// similar to $.extend; if only 1 hash in arguments ep will extend with this hash
		extend: function() {
			return extend.apply( this, arguments );
		},
		// similar to $.mixin; if only 1 hash in arguments ep will mixin with this hash
		mixin: function() {
			return mixin.apply( this, arguments );
		}
	});

	ep.extend({
		// find a key by the givn value in a hash
		inHash: function( val, obj ){
			var ret;
			if( !$.isPlainObject(obj) ){
				return ret;
			}
			$.each( obj, function( key, value ){
				if( value === val ){
					ret = key;
					return false;
				}
			});
			return ret;
		},
		// return array of keys, values or keys and values from hash
		hashToArray: function( obj, type ){
			var keys = [],
				values = [];
			$.each( obj, function( key, value ){
				keys.push( key );
				values.push( value );
			});
			return (/^k/i).test(type) ? keys : ( (/^v/i).test(type) ? values : [keys,values] );
		},
		// build a hash from 2 arrays (keys values)
		arrayToHash: function( keys, values ){
			if( !values ){
				values = keys;
				keys = [];
			}

			var hash = {};
			$.each( values, function( i, elm ){
				hash[ keys[i] || i ] = elm;
			});
			return hash;
		},
		// add item(s) to an array or array-like object at an indexed position without overwrite existing items
		insert: function( target, insert, index ){
			index = typeof index !== 'number' ? target.length : index;

			var end = splice.call( target, index, target.length-index );

			if( $.isArray(insert) ){
				push.apply( target, insert );
			}
			else if( typeof insert === 'object' && typeof insert.length === 'number' ){
				$.merge( target, insert );
			}
			else{
				push.call( target, insert );
			}

			return $.merge( target, end );
		},
		// escape string to use it for RegExp
		escapeExpStr: function( string, except ){
			return string.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
				if(except && except.indexOf(ch) !== -1){
					return ch;
				}
				return "\\" + ch;
			});
		},
		// scrunch a string [ fulltext... , full...text , ...fulltext ]
		scrunch: function( string, length, options ) {
			if( typeof length === 'object' ){
				options = length;
				length = 30;
			}
			var opt = $.extend({
					length:	length || 30,
					ratio:		1,
					cut:	true,
					spacer:	'...'
				},options);
			opt.length -= opt.spacer.length;

			var strLength = string.length;

			if( strLength <= opt.length ){
				return string;
			}
			else if( opt.ratio===0 ) {
				string = string.substr( strLength-opt.length, opt.length );
				if(!opt.cut) {
					string = string.substr( string.indexOf(" "), string.length );
				}
				return opt.spacer + string;
			}
			else if( opt.ratio===1 ){
				string = string.substr( 0, opt.length );
				if(!opt.cut) {
					string = string.substr( 0, string.lastIndexOf(" ")+1 );
				}
				return string + opt.spacer;
			}
			else{
				var aLength = Math.floor( opt.length * opt.ratio ),
					bLength = Math.floor( opt.length * (1-opt.ratio) ),
					cLength = opt.length - (aLength + bLength);

				return string.substr( 0, aLength ) + opt.spacer + string.substr( Math.max( 0, strLength-bLength-cLength ), strLength );
			}
		},
		// reverse a string or number or array
		reverse: function( string ) {
			if( $.isArray(string) ){
				return string.reverse();
			}

			var ret = '';
			(string+'').replace(/(.{1})/g,function(s) {
				ret = s+ret;
			});
			return ret;
		},
		// remove xml/xhtml/html tags
		stripTags: function( string ) {
			return string.replace(/<.[^<>]*?>/g, ' ');
		},
		// hash to address functions as global object in inline scripts
		local:{}
	});

	// Expose ep to the global object
	window.ep = ep;

});