/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.event.fix
 */

jQuery.plugin( 'jQuery.event.fix', function( $, undefined ){

	var _fix = $.event.fix;

	$.extend( true, $.event, {
		fix: function( event ) {
			// run  the original fix method
			event = _fix.call( $.event, event );

			// get the original event
			if( !event.originalEvent ){
				return event;
			}

			var originalEvent = event.originalEvent;

			// Add originalTarget, if necessary
			if ( !event.originalTarget ) {
				event.originalTarget = originalEvent.srcElement || event.target;
			}

			// be sure touches isn't undefined
			event.touches = originalEvent.touches===undefined ? [] : originalEvent.touches;

			// Calculate pageX/Y if missing and touches[0] available
			if ( event.touches.length ) {
				event.pageX = event.touches[0].pageX;
				event.pageY = event.touches[0].pageY;
			}

			// Calculate pageX/Y and touches[0] if missing and touch available
			if ( originalEvent.touch && !event.touches.length ) {
				event.touches={
					0:{
						pageX:originalEvent.touch.pageX,
						pageY:originalEvent.touch.pageY
					},
					length:1
				};
				event.pageX = originalEvent.touch.pageX;
				event.pageY = originalEvent.touch.pageY;
			}

			return event;
		}
	});

});