/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.event.special.dom
 */

jQuery.plugin( 'jQuery.event.special.dom', function( $, undefined ){

	/*
	 * @description		special native bindable dom events
	 */
	var emHelper = {},
		// interval handle
		domHandle = new function() {
			var self = this;

			this.vHandles = {};
			this.emHandles = {};

			this._interval = setInterval(function(){
				// show, hide, toggle, resize
				$.each( self.vHandles, function(){
					var now,
						before;
					if( this.type === 'resize' ) {
						now = [ this.element.width(), this.element.height() ];
						before = this.now;
					}
					else{
						now = !$.expr.filters.hidden(this.target);
						before = this.now;
					}

					this.now = now;
					this.before = before===undefined ? now : before;

					switch( this.type ){
						case 'show':
							if( this.now && !this.before ) {
								this.element.trigger(this.type);
							}
						break;
						case 'hide':
							if( !this.now && this.before ) {
								this.element.trigger(this.type);
							}
						break;
						case 'toggle':
							if( this.now !== this.before ) {
								this.element.trigger(this.type);
							}
						break;
						case 'resize':
							if( this.now[0] !== this.before[0] || this.now[1] !== this.before[1] ) {
								this.element.trigger(this.type);
							}
						break;
					}
				});
				// emchange
				if( emHelper.target ){
					var now = emHelper.target.offsetWidth/100,
						before = emHelper.now;

					emHelper.now = now;
					emHelper.before = before===undefined ? now : before;
					emHelper.delta = emHelper.now - emHelper.before;

					if( emHelper.now != emHelper.before ){
						$.each( self.emHandles, function(){
							var event = new $.Event(this);
								event.emSize = emHelper.now;
								event.emDelta = emHelper.delta;

							this.element.trigger(event);

						});
					}
				}

			},50);
		};

	// show, hide, toggle, resize, emchange
	$.each(['show', 'hide', 'toggle', 'resize', 'emchange'], function( i, name ) {
		$.event.special[ name ] = {
			add: function( bind ) {
				if( this===window && name==='resize' ){
					return;
				}
				bind.target = this;
				bind.element = $(this);
				domHandle[ name==='emchange' ? 'emHandles' : 'vHandles' ][bind.guid] = bind;
			},
			remove: function( unbind ) {
				if( this===window && name==='resize' ){
					return;
				}
				delete( domHandle[ name==='emchange' ? 'emHandles' : 'vHandles' ][unbind.guid] );
			}
		};
	});
	// build check node for emchange
	$.ready(function(){
		emHelper.node = $('<div />').css({left:'-100em',position:'absolute',width:'100em'}).prependTo('body')[0];
	});

});