/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.event.special.mouse
 */

jQuery.plugin( 'jQuery.event.special.mouse', function( $, undefined ){

	var html = $('html'),
		mousedown = function(originalBind){
			var ret = mousedown[originalBind.id] = function(event){
					html.bind( 'mousemove', mousemove(originalBind) )
						.bind( 'mouseup', mouseup(originalBind) );

					mousemove[originalBind.id].pageX = event.pageX;
					mousemove[originalBind.id].pageY = event.pageY;
				};
			return ret;
		},
		mousemove = function(originalBind){
			var ret = mousemove[originalBind.id] = function(event){

					event.type = originalBind.type;
					event.target = originalBind.target;
					event.data = originalBind.data;

					event.dragDeltaX = event.pageX - mousemove[originalBind.id].pageX;
					event.dragDeltaY = event.pageY - mousemove[originalBind.id].pageY;
					event.dragDelta  = delta  = Math.sqrt( Math.pow(event.dragDeltaX,2) + Math.pow(event.dragDeltaY,2) );

					mousemove[originalBind.id].pageX = event.pageX;
					mousemove[originalBind.id].pageY = event.pageY;

					return originalBind.originalHandler.apply(originalBind.target,arguments);
				};
			return ret;
		},
		mouseup = function(originalBind){
			var ret = mouseup[originalBind.id] = function(event){
					html.unbind( 'mousemove', mousemove[originalBind.id] )
						.unbind( 'mouseup', mouseup[originalBind.id] );

					delete(mousemove[originalBind.id]);
					delete(mouseup[originalBind.id]);
				};
			return ret;
		},
		mousewheel = function(originalBind){
			var ret = mousewheel[originalBind.id] = function(event){

					var orgEvent = event.originalEvent;

					event.wheelDelta  = orgEvent.wheelDelta  ? orgEvent.wheelDelta  / 120 : orgEvent.detail ? orgEvent.detail*3 : 0;
					event.wheelDeltaX = orgEvent.wheelDeltaX ? orgEvent.wheelDeltaX / 120 : 0;
					event.wheelDeltaY = orgEvent.wheelDeltaY ? orgEvent.wheelDeltaY / 120 : event.wheelDelta;

					if( $.browser.opera ) {
						event.wheelDelta  *= -1;
						event.wheelDeltaX *= -1;
						event.wheelDeltaY *= -1;
					}

					event.type = originalBind.type;

					return originalBind.originalHandler.apply(this,arguments);
				};
			return ret;
		};

	$.extend($.event.special,{
		/*
		 * @function		mousedrag
		 * @description		bind handler to element on drag
		 */
		mousedrag:{
			add: function( bind ) {
				bind.target = this;
				bind.originalHandler = bind.handler;

				$(this).bind( 'mousedown', mousedown(bind) );
			},

			remove: function( unbind ) {
				$(this).unbind( 'mousedown', mousedown[unbind.id] );
				html.unbind( 'mousemove', mousemove[unbind.id] );

				delete(mousedown[unbind.id]);
				delete(mousemove[unbind.id]);
			}
		},
		mousewheel:{
			add: function( bind ) {
				bind.originalHandler = bind.handler;

				if( $.browser.mozilla ){
					$(this).bind('DOMMouseScroll',mousewheel(bind),false);
				}
				else {
					bind.handler = mousewheel(bind);
				}
			},

			remove: function( unbind ) {
				if( $.browser.mozilla ){
					$(this).unbind('DOMMouseScroll',mousewheel[bind.id],false);
					delete( mousewheel[unbind.id] );
				}
				else{
					delete( mousewheel[unbind.id] );
				}
			}
		}
	});

});