use Test::Simple tests => 26;
use strict;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Cartridge::API::Constants qw( STATE_INSTALLED );
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( InsertCartridge DeleteCartridge GetPKeyCartridgesByPackage );
use DE_EPAGES::Shop::API::Widget qw( AddWidget );
use DE_EPAGES::Core::API::Error qw( ExistsError );

my %Vars;

## no critic qw( ProhibitMagicNumbers )
sub Main {
    ConnectTestSite();
    Init();

    #Test WidgetGroup
    #Instance
    my $WidgetGroup = $Vars{'WidgetGroups'}->[0];
    ok( $WidgetGroup->id, 'Create ' . $WidgetGroup->alias );
    ok( $WidgetGroup->instanceOf('WidgetGroup'), $WidgetGroup->alias . ' is instance of class WidgetGroup' );

    #Test WidgetType
    #Instance
    my $WidgetType = $Vars{'WidgetTypes'}->[0];
    ok( $WidgetType->id, 'Create ' . $WidgetType->alias );
    ok( $WidgetType->instanceOf('WidgetType'), $WidgetType->alias . ' is instance of class WidgetType' );

    #Attribute WidgetGroup
    my $WidgetGroupAlias = $WidgetType->get('WidgetGroup')->alias;
    ok( $WidgetGroupAlias eq $Vars{'WidgetGroups'}->[0]->alias,
        $WidgetType->alias . ' is member of '. $WidgetGroupAlias );

    #Attribute Template
    ok( $WidgetType->get('Template') eq 'TestTemplate0', 'WidgetType template is TestTemplate0' );
    $WidgetType->set({ 'Template' => 'TestTemplate1' });
    ok( $WidgetType->get('Template') eq 'TestTemplate1', 'WidgetType template is TestTemplate1' );

    #Attribute IsCustomizable
    ok( !$WidgetType->get('IsCustomizable'), $WidgetType->alias . ' is not customizable' );
    $WidgetType->set({ 'IsCustomizable' => 1 });
    ok( $WidgetType->get('IsCustomizable'), $WidgetType->alias . ' is customizable' );

    #Test Widget
    my $WidgetFolder = $Vars{'User'}->folder('MBOWidgets');
    foreach my $Widget (@{$WidgetFolder->children}){
        my $WAlias = $Widget->alias;
        #Instance
        ok( $Widget->id, 'Create widget ' . $WAlias  );
        ok( $Widget->instanceOf('Widget'), $WAlias  . ' is instance of class Widget' );

        #Type
        my $WTAlias = $Widget->get('WidgetType')->alias;
        ok( $WTAlias eq $Vars{'WidgetTypes'}->[0]->alias, $WAlias .' is of type ' . $WTAlias );

        #User
        my $UAlias = $Widget->get('User')->alias;
        ok( $UAlias eq $Vars{'User'}->alias, $WAlias . ' belongs to user ' . $UAlias );
    }

    #Test API
    #AddWidget
    my $Widget = AddWidget( $Vars{'User'}, $Vars{'WidgetTypes'}->[0] );
    ok( !ExistsError() && scalar grep( {$_ == $Widget->id} @{$Vars{'User'}->folder('MBOWidgets')->findChildren} ),
        $Widget->alias . ' was added to ' . $Widget->get('User')->alias );

    eval { AddWidget( $Vars{'User'}, $Vars{'WidgetTypes'}->[1] ) };
    ok( ExistsError() , 'TestShop1 has not the feature for this widget type ' . $Vars{'WidgetTypes'}->[1]->alias );

    #ChangePosition
    $Widget->changePosition(25);
    my $Position = $Widget->get('Position');
    ok( $Position == 30 , 'Widget->changePosition(25) - new position: ' . $Position );

    #SetAttributes
    $Widget->set({ 'Position' => 25 });
    ok( $Widget->get('Position') == 25 , 'Widget->set' );

    #RemoveWidget
    $Widget->delete;
    ok( !scalar grep({$_ == $Widget->id}@{$Vars{'User'}->folder('MBOWidgets')->findChildren} ), 'Widget->delete' );

    Cleanup();
    DisconnectSite();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};

    foreach my $Path (qw ( /Shops/TestShop1
                           /WidgetGroups/TestWidgetGroup0
                           /WidgetGroups/TestWidgetGroup1
                           /WidgetTypes/TestWidgetType0
                           /WidgetTypes/TestWidgetType1
                           /Features/TestFeature0
                           /Features/TestFeature1)){
        LoadObjectByPath($Path)->delete if ExistsObjectByPath($Path);
    }

    DeleteCartridge( $_ ) foreach @{GetPKeyCartridgesByPackage('C1::C1')};

    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();

    my $System = LoadRootObject();

    #Create shop
    #/Shops/TestShop1
    my $Shop = $System->folder('Shops')->insertChild( 'Shop', { 'Alias' => 'TestShop1' } );

    my $CartridgeID = InsertCartridge( { Package => 'C1::C1', Version => 25, InstallState => STATE_INSTALLED, DBStructureVersion => 25} );

    #/Features/TestFeature0
    #/Features/TestFeature1
    $Vars{'Features'} = [];
    foreach(0..1){
        push @{$Vars{'Features'}},
            $System->folder('Features')->insertChild( 'Feature', {
                Alias => 'TestFeature'.$_,
                CartridgeID => $CartridgeID,
                MaxValue => 1
            } );
    }
    $Shop->featureMaxValue( $Vars{'Features'}->[0], 1 );

    #Create two widget groups
    #/WidgetGroupFolder/TestWidgetGroup0
    #/WidgetGroupFolder/TestWidgetGroup1
    my $WidgetGroupFolder = $System->folder('WidgetGroups');

    $Vars{'WidgetGroups'} = [];
    foreach(0..1){
      push @{$Vars{'WidgetGroups'}},$WidgetGroupFolder->insertChild('WidgetGroup',{'Alias'=>'TestWidgetGroup'.$_});
    }

    #Create two widget types
    #/WidgetTypes/TestWidgetType0
    #/WidgetTypes/TestWidgetType1
    $Vars{'WidgetTypes'} = [];
    foreach(0..1){
        push @{$Vars{'WidgetTypes'}},
            $System->folder('WidgetTypes')->insertChild( 'WidgetType', {
                                                           'Alias' => 'TestWidgetType'.$_,
                                                           'WidgetGroup' => $Vars{'WidgetGroups'}->[0],
                                                           'Feature' => $Vars{'Features'}->[$_],
                                                           'Template' => 'TestTemplate0',
                                                           'IsCustomizable' => 0,
                                                          });
    }
    #Create user
    $Vars{'User'} = $Shop->folder('Users')->insertChild( 'User', { 'Alias' => 'TestUser1', 'Password' => '' } );

    #Create widget folder and add some widgets to user
    my $WidgetFolder = $Vars{'User'}->insertChild( 'Object', { 'Alias' => 'MBOWidgets' } );
    foreach(1..3){
        $WidgetFolder->insertChild( 'Widget', {
            'WidgetType' => $Vars{'WidgetTypes'}->[0],
            'Position' => $_ * 10,
        } );
    }

    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
