use Test::More tests => 5;
use strict;
use Config;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Core::API::InlineJavaServer qw( ExistsRunningInlineJavaServer StartInlineJavaServer StopInlineJavaServer RestartInlineJavaServer GetInlineJavaConfig CreateJavaObject );

RunScript( Sub => sub {

    StopInlineJavaServer();

    print "\n> Test Config.\n";
    my $hConfig = GetInlineJavaConfig();
    my $ConfigValues =    $hConfig->{'BIND'}
                       and $hConfig->{'HOST'}
                       and $hConfig->{'PORT'}
                       and $hConfig->{'J2SDK'}
                       and $hConfig->{'DIRECTORY'}
                       and $hConfig->{'STARTUP_DELAY'}
                       ? 1 : 0;
    ok( $ConfigValues, "got config values." );

    print "\n> Start server.. Is it running?\n";
    StartInlineJavaServer();
    sleep(2);
    ok( ExistsRunningInlineJavaServer(), "server is running" );

    print "\n> Stop server.. Is it running?\n";
    StopInlineJavaServer();
    sleep(2);
    ok( !ExistsRunningInlineJavaServer(), "server is not running" );

    print "\n> Create java object and call a method.. server should start automatically!\n";
    my @Params = ('bla', 'bli', 'blubb');
    my $PDFConverter = CreateJavaObject({
        'ClassName' => 'JavaServerTest',
        'ClassPath' => $ENV{'EPAGES_CARTRIDGES'}. '/DE_EPAGES/Core/t/JavaLib/JavaServerTest.jar',
        'Params'    => [ \@Params ]
    });

    my $Item = eval { $PDFConverter->getItem(2) };
    ok( (!ExistsError() && $Item eq $Params[2]) ? 1 : 0, "called object method successfully" );

    print "\n> Restart server.\n";
    my $PID_0 = GetJavaPID($hConfig);
    RestartInlineJavaServer();
    sleep(5);
    my $PID_1 = GetJavaPID($hConfig);
    ok((ExistsRunningInlineJavaServer() && $PID_0 != $PID_1) ? 1 : 0,"restarted successfully");

    StopInlineJavaServer();

});

sub GetJavaPID {
    my ($hConfig) = @_;
    my $JavaPID = undef;

    if($Config{'osname'} eq 'MSWin32') {
        my @ProcessList = `netstat -no`;
        $JavaPID = (grep {/\s*tcp\s*\S*\:$hConfig->{PORT}/i} @ProcessList)[0];
    } else {
        my @ProcessList = `netstat -nlept`;
        $JavaPID = (grep {/\s*tcp.*\:$hConfig->{PORT}/i} @ProcessList)[0];
    }
    $JavaPID =~ s/.*\s(\d+).*$/$1/ if defined $JavaPID;

    return $JavaPID;
}
