/*
 * @charset			utf-8		(ÄÖÜ äöü ß)
 *
 * Copyright (c) 2006-2010, epages GmbH
 * All Rights Reservedojo.
 * epages.mimetype $Revision: 1.7.2.1 $
 */

dojo.provide('epages.mimetype');

(function(dojo) {

	dojo.declare('epages.Mimetype',null,{
	// summary: 		mimetype and filetype handling
	// description: 	privat class
	// TODO:			improve this class and make it to a public class
	//					by preserving the epages.mimetype API

		/*
		 * private
		 */
		 _MIMETYPE_MAP: {
			// image
			'image/png':																	['png' ],
			'image/jpeg':																	['jpe', 'jpeg', 'jpg' ],
			'image/gif':																	['gif' ],
			'image/bmp':																	['bmp' ],
			'image/vnd.microsoft.icon':														['ico' ],
			'image/tiff':																	['tif', 'tif' ],
			'image/svg+xml':																['svg', 'svgz' ],
			'image/x-win-bitmap':															['cur' ],
			// audio/video
			'audio/basic':																	['au'],
			'audio/mpeg':																	['mp3' ],
			'audio/mp4a-latm':																['m4p', 'm4b', 'm4a' ],
			'audio/x-wav':																	['wav' ],
			'audio/ogg':																	['ogg', 'oga', 'spx' ],
			'audio/flac':																	['flac' ],
			'audio/x-ms-wma':																['wma' ],
			'audio/x-pn-realaudio':															['ra','ram' ],
			'application/vnd.rn-realmedia':													['rm' ],
			'video/ogg':																	['ogv' ],
			'video/mp4':																	['mp4' ],
			'video/mpeg':																	['mpe', 'mpg', 'mpeg', 'mpga' ],
			'video/quicktime':																['qt', 'mov' ],
			'video/x-msvideo':																['avi' ],
			'video/x-flv':																	['flv' ],
			'video/x-ms-wmv':																['wmv'],
			// web
			'text/plain':																	['txt' ],
			'text/html':																	['htm', 'html', 'shtml', 'php', 'php4' ],
			'text/css':																		['css' ],
			'text/x-actionscript':															['as'],
			'application/javascript':														['js' ],
			'application/json':																['json' ],
			'application/xml':																['xml', 'plist' ],
			'application/x-shockwave-flash':												['swf' ],
			// archives
			'application/zip':																['zip' ],
			'application/gzip':																['gz' ],
			'application/x-tar':															['tar' ],
			'application/x-rar-compressed':													['rar' ],
			'application/x-msdownload':														['exe', 'msi' ],
			'application/vnd.ms-cab-compressed':											['cab' ],
			// adobe
			'application/pdf':																['pdf' ],
			'image/vnd.adobe.photoshop':													['psd' ],
			'application/postscript':														['ai', 'eps', 'ps' ],
			// ms office
			'application/msword':															['doc' ],
			'application/x-dot':															['dot' ],
			'application/rtf':																['rtf' ],
			'application/vnd.ms-excel':														['xls','xla' ],
			'application/vnd.ms-powerpoint':												['ppt' ],
			'application/vnd.openxmlformats-officedocument.wordprocessingml.document': 		['docx'],
			'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':			['xlsx'],
			//
			'application/x-director':														['cdr']
		},

		_MIMETYPE_GROUPS: {
			'*/*':				['*/*' ],
			'icon/*':			['image/vnd.microsoft.icon', 'image/x-win-bitmap' ],
			'image/*':			['image/png', 'image/jpeg', 'image/gif' ],
			'document/*':		['application/pdf', 'application/msword', 'application/vnd.ms-excel', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' ],
			'audio/*':			['audio/mpeg', 'audio/x-ms-wma', 'audio/ogg', 'audio/flac', 'audio/x-ms-wma' ],
			'video/*':			['video/x-msvideo', 'video/mp4', 'video/mpeg', 'video/x-ms-wmv', 'video/x-flv' ]
		},

		_MIMETYPE_PREVIEWABLE: 	['image/png', 'image/jpeg', 'image/gif', 'image/vnd.microsoft.icon' ],

		_ICON_MAP: {
			'ico_l_image':		['image/png','image/jpeg','image/gif','image/bmp','image/vnd.microsoft.icon','image/tiff','image/svg+xml','image/x-win-bitmap'],
			'ico_xl_box':		[this._UNKNOW,'text/x-actionscript','audio/basic','audio/x-wav','audio/mpeg','audio/x-ms-wma','audio/ogg','audio/flac','application/msword','application/x-dot','application/vnd.ms-excel','application/vnd.ms-powerpoint','text/plain','text/css','application/javascript','application/json','application/xml'],
			'ico_xl_file_avi':	['video/x-msvideo','video/mpeg','video/mp4'],
			'ico_xl_file_wmv':	['video/x-ms-wmv'],
			'ico_xl_file_swf':	['video/x-flv','application/x-shockwave-flash'],
			'ico_xl_file_mov':	['video/quicktime','video/ogg'],
			'ico_xl_file_rm':	['application/vnd.rn-realmedia'],
			'ico_xl_file_ram':	['audio/x-pn-realaudio'],
			'ico_xl_file_dcr':	['application/x-director'],
			'ico_xl_file_pdf':	['application/pdf'],
			'ico_xl_file_htm':	['text/html'],
			'ico_m_folder':		['application/zip','application/x-rar-compressed','application/gzip','application/x-tar']
		},

		_UNKNOW: 'application/octet-stream',

		_mimes: null,

		/*
		 * public
		 */

		/*
		 * init instance
		 */
		constructor: function( mime ) {
			this.setMime( mime || this._UNKNOW )
		},

		/*
		 * methods
		 */
		setBySuffix: function( suffix ) {
			suffix = suffix.toLowerCase();
			this._mime = this._UNKNOW;

			var map = this._MIMETYPE_MAP;

			for( var i in map ) {
				if( dojo.indexOf(map[i], suffix)>=0 ) {
					this._mime = i;
					break;
				}
			};

			return this;
		},

		setMime: function( mime ) {
			this._mime = mime;
			return this;
		},

		getMime: function() {
			return this._mime;
		},

		mimesByGroup: function( group ) {
			return this._MIMETYPE_GROUPS[group];
		},

		isGroup: function( group ) {
			var mimesGroup = this._MIMETYPE_GROUPS[group];

			if( group=='*/*' ) {
				return true;
			}
			else if(!mimesGroup) {
				return false;
			}
			else {
				if( dojo.indexOf(mimesGroup, this._mime)>=0 ) {
					return true;
				}
			}

			return false;
		},

		isPreviewable: function() {
			var prev = this._MIMETYPE_PREVIEWABLE;

			if( dojo.indexOf(prev, this._mime)>=0 ) {
				return true;
			}

			return false;
		},

		getIcon: function() {

			var map = this._ICON_MAP;
			for( var i in map ) {
				if( dojo.indexOf(map[i], this._mime)>=0 ) {
					return i;
				}
			}

			return 'ico_xl_box';
		}

	});

	var _mimetype = new epages.Mimetype();

	epages.mimetype = {
		mimeBySuffix: function( suffix ) {
		// summary: get mimetypeby suffix
			return _mimetype.setBySuffix( suffix );
		},
		isPreviewable: function( mime ) {
		// summary: to check is the givn mimetype previewable
			return _mimetype.setMime( mime ).isPreviewable();
		},
		isGroup: function( mime , group ) {
		// summary: to check is the givn mimetype in a mimetype group
			return _mimetype.setMime( mime ).isGroup( group );
		},
		getGroup: function( group ) {
		// summary: return a group of mimetypes
			return _mimetype._MIMETYPE_GROUPS[group] || ['*/*'];
		},
		isGroupBySuffix: function( suffix , group ) {
		// summary: to check is the givn fileSuffix in a mimetype group
			return _mimetype.setBySuffix( suffix ).isGroup( group );
		},
		getIcon: function( mime ) {
		// summary: returns a SpriteIconClass by the givn mimetype
			return _mimetype.setMime( mime ).getIcon();
		}
	};

})(dojo);