/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.dict
 */

jQuery.plugin( 'jQuery.dict', ['jQuery.mixin','jQuery.scope'], function( $, undefined ){

	// extend the dependencies controller
	var dependSettings = $.dependSettings,
		dictSplit = function(name){
			var ret = {
					name: name
				},
				tmp = name.split(':');

			ret.dictRegion	= tmp[1] ? tmp.shift() : $.dictSettings.region,
			ret.dictName	= tmp.join(':');
			ret.dict		= ret.dictRegion+':'+ret.dictName;

			return ret;
		};

	if( !dependSettings.dictRoot )
		dependSettings.dictRoot = $.dependSettings.pluginRoot;

	$.dependHandle.dict = {
		ajax: function( name ){
			var o = dictSplit(name),
				ajax = {
					url:		dependSettings.tmplRoot+'/'+o.dictName.replace(/\./g,'/')+'.'+o.dictRegion+(dependSettings.minify?'.min':'')+'.json',
					dataType:	dependSettings.jsonp ? 'jsonp' :'json',
					type:		'get',
					asnyc:		true,
					cache:		true,
					context:	o,
					success: function( data ){
						jQuery.dependLoad( this.name, 'dict', data );
						jQuery.dependReady( this.name, 'dict' /*, data */ );
					}
				};

			return dependSettings.jsonp ?
				$.extend(ajax,{
					dataType:		'jsonp',
					jsonp:			dependSettings.jsonp!==true ? dependSettings.jsonp : undefined,
					jsonpCallback:	'jsonp.dependHandle.dict('+o.dict+')'
				}) :
				ajax;
		},
		isLoad: function( name ){
			return !!$.dict[ dictSplit(name).dict ];
		},
		isReady: function( name ){
			return !!$.dict[ dictSplit(name).dict ];
		},
		onLoad: function( name , data ){
			$.dict[ dictSplit(name).dict ] = data;
		},
		onReady:  $.noop	/* function( name , data ){} */
	};

	$.extend({
		dict: function( dict, data, options ) {

			if( !$.isArray(data) ) {
				options = data;
				data = [];
			}

			var o = $.extend( {}, $.dictSettings, options || {} ),
				fullName = o.region+':'+dict,
				args = [ $.dict[ fullName ] || {} ];

			$.each( data, function( i, name ){
				if( typeof name==='string' ){
					args.push( $.dict[ o.region+':'+name ] || {} );
				}
				else if( name instanceof $.Dictionary ){
					args.push( name.dict );
				}
				else if( $.isPlainObject(name) ){
					args.push( name );
				}
			});

			var dictData = $.dict[ fullName ] = args.length>1 ? $.mixin.apply( {}, args ) : args[0];

			return new $.Dictionary( dictData, o );
		},

		dictSettings: $.extend(
			{
				parser: [ /([^\{\$]{1}|^)\{([^\{\}]+)\}/g, 1 ],
				region: 'en'
			},
			$.config.dict
		),

		Dictionary: function( dict, settings ){
			this.dict = dict;
			this.settings = settings;
		}
	});

	$.extend( $.Dictionary.prototype, {

		translate: function( key ){
			var repl = this.dict[ key ];
			return repl===undefined ? key : repl;
		},

		parse: function( string ) {
			var dict = this.dict,
				index = this.settings.parser[1]+1,
				repl = function(){
					var repl = dict[ arguments[index] ],
						ret = '';
					// don't replace if undefined
					if( repl===undefined ){
						return arguments[0];
					}
					// set replace
					arguments[index] = repl;
					// build return
					for( var i=1,l=arguments.length-2 ; i<l ; i++ ){
						ret += arguments[i];
					}
					return ret;
				};
			return string.replace( this.settings.parser[0], repl );
		}

	});

	// add dict as dom manipulation method
	$.fn.extend({
		dictParse: function( dict, deep, options ){
			var elem = this;
			// parse DOM recursive
			if( typeof deep === 'boolean' ){
				if( deep===true )
					elem = this.contents();
			}
			else{
				options = deep;
				deep = false;
			}

			dict = $.dict( dict, options );

			elem.each(function(){
				$.each( this.childNodes, function( i, node ){
					if( this.nodeType===3 ){
						if( this.innerTEXT ){
							this.innerTEXT = dict.parse(this.innerTEXT);
						}
						else if( this.textContent ){
							this.textContent = dict.parse(this.textContent);
						}
						else if( this.nodeValue ){
							this.nodeValue = dict.parse(this.nodeValue);
						}
					}
				});
			});

			return this;
		}
	});

});