use Test::More;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::File  qw ( GetFileContent WriteFile DeleteFile );
use DE_EPAGES::Core::API::PerlTools qw(UsePackage);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::TLE::API::Template qw ( ProcessFileTemplate );

our $TEST_ONE;

local $| = 1;
RunScript( Sub => sub {
    Initialize();
    if ( defined $TEST_ONE ) {
        plan tests => 1;
        ok ( Test_Package($TEST_ONE ), $TEST_ONE );
    }
    else {
        my @Files = glob '*_t.tmpl';
        plan tests => (defined $TEST_ONE ? 1 : scalar @Files);

        foreach my $File ( @Files ) {
            if ($File =~ /^(.+)_t\.tmpl$/) {
                my $TestCase = $1;
                ok ( Test_Package($TestCase ), $TestCase );
            }
            else {
                die "invalid file name $File";
            }
        }
    }
    Cleanup();
});

sub ErrorDefinition {
    return {
        NO_EQUAL_LINE => "Output is different at line #AtLine\n      got: #Ist\n expected: #Exp"
    };
}

sub Test_Package {
    my ($i) = @_;
    my $Package = 'DE_EPAGES::TLE::t::tlevars';
    UsePackage( $Package );
    my $isContent;
    eval {
        no strict 'refs';  ##no critic qw( ProhibitNoStrict )
        my $Loops =  &{"$Package\:\:GetLoops"}($i);
        my $Vars  =  &{"$Package\:\:GetVars"}($i);

        $isContent = ProcessFileTemplate($i.'_t.tmpl', { %$Vars , %$Loops});
        my $rExContent = GetFileContent($i.'_e.tmpl');
        CompareContent($rExContent, \$isContent);
    };
    if (ExistsError()) {
        my $Error = GetError();
        note $Error->shortMessage();
        $Error->{'Vars'}->{'Test'} = $i;
        WriteFile($i.'_i.tmpl', \$isContent) if defined $isContent;
        return 0;
    } else {
        DeleteFile($i.'_i.tmpl');
    }
    return 1;
}

sub CompareContent {
    my ($rExpected, $rIs) = @_;

    my @aExpected = split /\n/, $$rExpected;
    my @aIs = split /\n/, $$rIs;
    for (my $i=0;$i < scalar @aExpected; $i++) {
        if ($aExpected[$i] ne $aIs[$i]) {
            Error('NO_EQUAL_LINE', {'AtLine' => $i + 1, 'Exp'=>$aExpected[$i], 'Ist'=> $aIs[$i] });
        }
    }
    if (scalar @aIs != scalar @aExpected) {
        Error('NO_EQUAL_LINES', {'CountLinesIs' => scalar @aIs, 'CountLinesExpected' => scalar @aExpected } );
    }
    return 1;
}

sub Initialize {
    Cleanup();
}

sub Cleanup {
    DeleteFile($_) foreach glob('*.ctmpl');
}

1;

