use Test::More tests => 12;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::InlineJavaServer qw( StopInlineJavaServer );
use DE_EPAGES::Core::API::PdfConverter qw( IsValidXhtml ConvertXhtml2Pdf );

RunScript( Sub => sub {

    StopInlineJavaServer();

    my $HtmlCode = '<html><head></head><body><p>blubb</p></body></html>';
    ok( IsValidXhtml($HtmlCode), "IsValidXhtml: valid xhtml" );

    $HtmlCode = '<html><head></head><body><p>blubb</body></html>';
    ok( !IsValidXhtml($HtmlCode), "IsValidXhtml: missing end tag" );

    $HtmlCode = '<html><head></head><body att=sddf><p>blubb</p></body></html>';
    ok( !IsValidXhtml($HtmlCode), "IsValidXhtml: missing quotes" );

    $HtmlCode = '<html><head></head><body att=sddf"><p>blubb</p></body></html>';
    ok( !IsValidXhtml($HtmlCode), "IsValidXhtml: odd number of qoutes (left one is missing)" );

    $HtmlCode = '<html><head></head><body att="sddf><p>blubb</p></body></html>';
    ok( !IsValidXhtml($HtmlCode), "IsValidXhtml: odd number of qoutes (right one is missing)" );

    $HtmlCode = '<html><head></head><body att="sddf"<p>blubb</p></body></html>';
    ok( !IsValidXhtml($HtmlCode), "IsValidXhtml: closing bracket is missing" );

    $HtmlCode = '<html><head></head><body att="sddf"><p>blubb</p></body>';
    ok( !IsValidXhtml($HtmlCode), "IsValidXhtml: </html> is missing" );

    $HtmlCode = '<head></head><body att="sddf"><p>blubb</p></body></html>';
    ok( !IsValidXhtml($HtmlCode), "IsValidXhtml:  <html> is missing" );

    $HtmlCode = '<p>blubb</p>';
    ok( IsValidXhtml($HtmlCode, {'isSnippet' => 1}), "IsValidXhtml: valid xhtml snippet" );

    $HtmlCode = '<p>blubb';
    ok( !IsValidXhtml($HtmlCode, {'isSnippet' => 1}), "IsValidXhtml: missing end tag at xhtml snippet" );

    $HtmlCode = '<html><head></head><body att="sddf"><p>blubb</p></body></html>';
    my $Output = ConvertXhtml2Pdf($HtmlCode);
    ok( $Output =~ /^%PDF/, "pdf document on valid xhtml" );

    $HtmlCode = '<html><head></head><body att="sddf><p>blubb<br>';
    $Output = ConvertXhtml2Pdf($HtmlCode);
    ok( $Output =~ /^%PDF/, "pdf document on invalid xhtml.. should be fixed automatically!" );

    StopInlineJavaServer();

});
