use strict;
use lib ($ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Core/Scripts');

use DE_EPAGES::Core::API::Script qw( RunScript );
use Perl::Critic;
use Test::More;

sub Main {

    my $critic = Perl::Critic->new(
        -profile  => 'NONE',
        -severity => 5
    );
    ok( defined $critic, "Perl::Critic initialized" );
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::ErrorHandling::RequireExistsError' );
    my @policies = $critic->policies();
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::ErrorHandling::RequireExistsError' }
    @policies;
    ok( $policyLoaded, "Policy loaded" );

    my @violations;
    my $sourceCode;

    ##no critic qw( ProhibitImplicitNewlines )
    $sourceCode = q{
        eval {
            1;
        };
        if ($a) {
            $a->b();
        }
    };

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, "ExistsError is missing") or diag(@violations);

    ##no critic qw( ProhibitImplicitNewlines )
    $sourceCode = q{
        eval {
            1;
        };
        if (ExistsError()) {
            $a->b();
        }
    };

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "ExistsError is there.") or diag(@violations);

    ##no critic qw( ProhibitImplicitNewlines )
    $sourceCode = q{
        eval {
            1;
        };
        if ($@) {
            $a->b();
        }
    };

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "\$@ is there.") or diag(@violations);

    $sourceCode = q{ eval $$rContent; print 'test'; };

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "do not test on stringy eval.") or diag(@violations);

}


RunScript( Sub => \&Main );
done_testing();