use strict;
use Test::More tests => 16;
use Scalar::Util qw(blessed);
use DE_EPAGES::DataCache::API::LRUBuffer;

# 1..2
my %TieCache;
my $TieCacheObject = tie %TieCache, 'DE_EPAGES::DataCache::API::LRUBuffer', { MaxCount => 3, MaxBytes => 2048 };
is( blessed $TieCacheObject, 'DE_EPAGES::DataCache::API::LRUBuffer', 'is lru buffer');

# test $infos
#   'CurrentBytes'  =>'0'
#   'CurrentCount'  =>'0'
#   'Misses'    =>'0'
#   'Hits'  =>'0'
#   'MaxBytes'  =>'1024'
#   'MaxCount'  =>'100'

# statistics
my $infos = $TieCacheObject->infos;
ok ($infos->{'MaxCount'} == 3, 'max count ok');
ok ($infos->{'MaxBytes'} == 2048, 'max bytes ok');

# partly used
ok (!exists $TieCache{1}, 'cache is still empty' );
$infos = $TieCacheObject->infos;
ok ($infos->{'Misses'} == 1, 'misses count exists');
ok (!defined $TieCache{1}, 'cache is still empty');
$infos = $TieCacheObject->infos;
ok ($infos->{'Misses'} == 2, 'misses counts defined');

$TieCache{1} = 'hallo epages';
$infos = $TieCacheObject->infos;
ok ($infos->{'CurrentCount'} == 1, 'current count');
ok (exists $TieCache{1}, 'cache item exists');
$infos = $TieCacheObject->infos;
ok ($infos->{'Hits'} == 1, 'count hists exists');
ok (defined $TieCache{1}, 'cache item is defined');
$infos = $TieCacheObject->infos;
ok ($infos->{'Hits'} == 2, 'count hists defined');
$TieCache{1}++;
$infos = $TieCacheObject->infos;
ok ($infos->{'Hits'} == 3, 'count hists on increment');

# test on full cache
$TieCache{$_} = $_ + 1 foreach qw (1 2 3 4 5 6 7 8 9);
$infos = $TieCacheObject->infos;
ok ($infos->{'CurrentCount'} == 3, 'max reached');
$TieCacheObject->setMaxCount(5);
$TieCache{$_} = $_ + 1 foreach qw (1 2 3 4 5 6 7 8 9);
$infos = $TieCacheObject->infos;
ok ($infos->{'MaxCount'} == 5, 'count changed');
ok ($infos->{'CurrentCount'} == 5, 'changed count works');

