use constant TESTS => 3;
use Test::More tests => TESTS;

use strict;
use DE_EPAGES::Core::API::File qw( ExistsFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Database::API::BackupMySQL qw( MySQLShowVariables MyCnfDefault );

sub Main {
    my ($dbi) = @_;

    my $hSettings = MySQLShowVariables(GetTestSite());
    my $MyCnf = MyCnfDefault($hSettings->{basedir});
    ok( ExistsFile($MyCnf), "MyCnfDefault: config file exists ($MyCnf)" );

}

RunScript(
    Debug => 1,
    Sub => sub {
        RunOnStore( Store => GetTestSite(), Sub => sub {
            my $dbi = GetCurrentDBHandle();
            SKIP: {
                # skip 1 test if not Sybase
                skip( "No MySQL DBD driver\n", TESTS ) unless $dbi->driverName eq 'mysql';

                ok( Init( $dbi ), 'Init' );
                Main( $dbi );
                ok( Cleanup( $dbi ), 'Cleanup' );
            }
        });
    }
);

sub Init {
    my ( $dbi ) = @_;
    Cleanup( $dbi );

    $dbi->do( 'CREATE TABLE dbitest( id INTEGER, name VARCHAR(50) NULL, dt DATETIME NULL )' );

    return $dbi->existsTableColumn( 'dbitest', 'name' );
}

sub Cleanup {
    my ( $dbi ) = @_;

    $dbi->do( 'DROP TABLE dbitest' ) if $dbi->existsTable( 'dbitest' );
    return 1;
}

1;
