/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.colorpicker
 *
 * @revision		$Revision: 1.4.4.3 $
 */

jQuery.plugin( 'ep.ui.colorpicker', {
	plugin:	['jQuery.tmpl', 'jQuery.metaparse', 'jQuery.ui.widget', 'jQuery.ui.draggable', 'ep.core', 'ep.dict', 'ep.color', 'ep.colorpicker', 'ep.fn.contextOrientation', 'ep.ui.input'],
	dict:	'ep.dict',
	tmpl:	'ep.ui.colorpicker'
}, function($){

/*
 * @dictionary		ep.dict
 *
 * @translation		{ColorCode}
 *					{Transparent}
 */
	var epColor = ep.color;

	$.widget( 'ep.ui.Colorpicker', {

		options: {
			autoOpen:			false,
			orientation:		'right',
			orientationAdjust:	[0,0],
			color:				'#fff',
			autoApply:			false,
			callback:			$.noop
		},

		_create: function(){
			var self = this;

			self.elem = self.element
				.bind( 'click', self.Proxy('open') )
				.sub(ep);

			self.originalColor = self._getColor().color;
		},

		_init: function(){
			var self = this,
				o = self.options;

			self.colorSelect();

			if( o.autoOpen ){
				self.open();
			}
			if( o.autoApply && self.applyButton ){
				self.applyButton.hide();
			}
		},

		_build: function(){
			if( !this.dialog ){
				var self = this,
					o = self.options,
					color = self._getColor(),
					colorpicker = self.colorpicker = new ep.Colorpicker({
						margin:		10,
						size:		200,
						hex:		color.hex,
						callback:	function( hex ){
							self._setColor( hex );
						}
					});

				// load template and push named tmpl elements to current instance
				$.extend( self, $.tmpl('ep.ui.colorpicker',{transparent:color.string=='transparent'}).tmplItem('elements') );

				self.dialog
					.hide()
					.appendTo('body')
					.dictParse('ep.dict',true)
					.draggable({
						containment:	'window',
						handle:			'.ep-uiColorpicker-head'
					})
					.find('.ep-metaparse')
						.metaparse();

				self.pickerBox
					.append(colorpicker.elem);
				self.closeButton
					.bind( 'click', self.Proxy('close') );
				self.previewBox
					.css('background','#'+o.color);
				self.hexInput
					.bind( 'changeValue', function(){
						var val = $(this).val();
						val = epColor.stringToName(val) || epColor.stringToHex(val);
						if( val ){
							self.colorSelect( val );
						}
					});
				self.transparentInput
					.bind('change',function(){
						self.colorSelect();
					});
				self.applyButton
					.bind('click',function(){
						self.colorApply();
						self.close();
					});

				self._init();
			}
		},

		_getColor: function( color ){
			color = color || this.options.color;

			var ret =
				(this.transparentInput && this.transparentInput.is(':checked')) || color=='transparent' ?
				{
					hex:	'ffffff',
					name:	'transparent'
				} :
				{
					hex:	epColor.stringToHex( color ),
					name:	epColor.stringToName( color )
				};

			ret.string = ret.name || '#'+(ret.hex || 'ffffff');

			return ret;
		},

		_setColor: function( color ){
			var self = this,
				o = self.options;

			o.color = color = self._getColor(color).string;

			self.elem.trigger('colorSelect',[color]);
			if( this.previewBox ){
				this.previewBox.css('background',color);
			}
			if( this.hexInput ){
				this.hexInput.val(color);
			}
			if( o.autoApply ){
				self.colorApply(color);
			}
		},

		colorSelect: function( color ){
			color = this._getColor(color);
			if( color ){
				this._setColor(color.string);
				if( this.colorpicker ){
					this.colorpicker.setHex( color.hex, true );
				}
			}
		},

		colorApply: function( color ){
			color = this._getColor(color).string;
			if( color ){
				this.originalColor = this.options.color = color;
				this.options.callback(color);
				this.elem.trigger('colorApply',[color]);
			}
		},

		colorReset: function(){
			this.elem.trigger('colorRest',[this.originalColor]);
			this._setColor( this.originalColor );
		},

		open: function(){
			var self = this,
				o = self.options;

			// close other colorpicker
			$('.ep-uiColorpicker-dialog').hide();

			self._build();
			self.dialog
				.show()
				.contextOrientation( self.elem, o.orientation, o.orientationAdjust );
			
			this.elem.trigger('open');
		},

		close: function(){
			if( this.dialog ){
				this.dialog.hide();
				this.elem.trigger('close');
			}
		},
		
		destroy: function(){
			if( this.dialog ){
				this.dialog.remove();
			}
			this.Inherited(arguments);
		}

	});

});