/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Duale licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @core			jQuery JavaScript Library v1.4.4 + SubClass pre build-in from jQuery v1.5b
 *
 * @desc			jQuery.core.sdk includes
 *					- .sub			In chaining subclass switch
 *					- jQuery.config		Configuration vars
 *					- jQuery.debug		Base property for debug tools
 *					- jQuery.plugin		Dependencies controller
 *					- jQuery.provide	Dependencies controller
 *					- jQuery.ready		Dependencies controller
 *					- jQuery.ready		DOM ready
 */

(function( jQuery, undefined ){

		// save original method
	var ready = jQuery.ready,

		// config from meta tag
		metaConfig = jQuery('meta[name=jQueryConfig]').attr('content'),

		// jQuery.config
		config = jQuery.extend( true, {}, window.jQueryConfig, (metaConfig ? eval('('+metaConfig+')') : undefined) ),

		// type of resolve deferred on done
		doneExp = /^(plugin|script)$/,

		// type of dependencies ready data
		typeExp = /^(string|object|array)$/,

		// type of plugin dependencies
		pluginExp = /(string|array)/,

		// count for ready calls
		readyCount = 0,

		// name of the property set to deferred object whether dependHandle is called
		isDependHandled = '--depend--controlled--data--',

		// settings of dependencies controller
		dependSettings,

		// type groups of the deferred objects
		dependStack,

		// handles of the dependencies controller
		dependHandle,

		// add dependencies to control
		dependControl = function( depends, callback, type, name ){
			if( doneExp.test(type) )
				var selfDeferred = dependStack[ type ][ name ] = (dependStack[ type ][ name ] || dependSetHandled(jQuery.Deferred()));

			var objType = jQuery.type(depends);
			// if no dependencies
			if( objType==='function' ){
				callback = depends;
				depends = {};
			}

			// if only plugins dependent
			if( pluginExp.test(objType) )
				depends = { plugin: depends };

			// stack of deferred object, first object is jQuery to receive it as argument in .done() callbacks
			var deferredStack = [jQuery];
			// add dependencies to deferredStack
			jQuery.each( depends, function( dependType, dependTypeDepends ){
				if( dependType == 'provide' )
					dependType = 'script';

				var dependTypeStack = dependStack[ dependType ] = dependStack[ dependType ] || {};

				jQuery.each( jQuery.isArray(dependTypeDepends) ? dependTypeDepends : [dependTypeDepends], function( i, depend ){
					deferredStack.push(
						dependHandleCall(
							dependType,
							depend,
							(dependTypeStack[ depend ] = dependTypeStack[ depend ] || jQuery.Deferred())
						)
					);
				});
			});

			// when deferredStack done
			return jQuery.when
				.apply( jQuery, deferredStack )
				.done(function(){
					if( callback )
						callback(jQuery);
					if( selfDeferred )
						selfDeferred.resolve();
				});
		},

		dependSetHandled = function( deferred ){
			deferred[ isDependHandled ] = !0;
			return deferred;
		},

		// call the registered dependHandle
		dependHandleCall = function( dependType, depend, deferred ){
			var dependTypeHandle = dependHandle[ dependType ];
			if( !deferred[ isDependHandled ] && !deferred.isResolved() && dependTypeHandle ){
				dependTypeHandle( depend, dependSetHandled(deferred) );
			}

			return deferred;
		},

		// deferred object which is already resolved
		ResolvedDeferred = dependSetHandled(jQuery.Deferred()).resolve(jQuery),

		// deferred object for Document Object Model ready state
		DOMReadyDeferred = dependSetHandled(jQuery.Deferred()),

		// when object for Document Object Model ready state
		DOMReayWhen = DOMReadyDeferred.promise();

	// bind deferred on DOM ready event
	jQuery.fn.ready(function(){
		DOMReadyDeferred.resolve(jQuery);
	});

	// run config for ajax
	jQuery.extend( jQuery.ajaxSettings, config.ajax );

	// run config for noConflict mode
	if( config.noConflict ){
		var alias = jQuery.noConflict();
		if( typeof config.noConflict === 'string' )
			window[ config.noConflict ] = alias;
	}

	jQuery.fn.extend({
		// .jquerysdk --> Version
		jquerysdk: '1.0',
		// .sub() --> provide subclass switch as prototype method
		sub: function( jQuerySubclass ){
			return jQuerySubclass( this );
		}
	});

	jQuery.extend({
		// jQuery.config
		config: config,

		// jQuery.debug
		debug: {
		// jQuery.debug.warn()
			warn: window.console && jQuery.isFunction(console.warn) ? function( thrown ){ console.warn( thrown ) } : jQuery.noop
		},

		// jQuery.dependSettings
		dependSettings: (dependSettings = jQuery.extend(
			{
				minify:			false,
				jsonp:			false,
				pluginRoot:		'script'
			},
			config.depend
		)),

		// jQuery.dependStack
		dependStack: (dependStack = {
			plugin:	{
				'jQuery.core.original':	ResolvedDeferred,
				'jQuery.core.sdk':		ResolvedDeferred
			},
			script:	{},
			DOM:	{
				'true':			DOMReadyDeferred,
				'false':		ResolvedDeferred
			},
			// raw depends for debug
			_plugin: {},
			_script: {}
		}),

		// jQuery.dependHandle
		dependHandle: (dependHandle = {
		/* script will not load from anywhere, so the dependController handle it on provide a script
			script: function( depend, deferred ){

			},
		*/

		/* deferred objects arw already defined
			DOM: function( depend, deferred ){

			},
		*/

			plugin: function( depend, deferred ){
				var ajax = {
						url:		dependSettings.pluginRoot+'/'+depend.replace(/\./g,'/')+(dependSettings.minify?'.min':'')+'.js',
						dataType:	'script',
						type:		'get',
						asnyc:		true,
						cache:		true
					};

				jQuery
					.ajax(
						dependSettings.jsonp ?
						jQuery.extend(ajax,{
							dataType:		'jsonp',
							jsonp:			jQuery.type(dependSettings.jsonp)==='string' ? dependSettings.jsonp : undefined,
							jsonpCallback:	'jsonp.depend.plugin('+depend+')'
						}) :
						ajax
					);
			}
		}),

		// jQuery.dependRegister() --> register an handle for dependencies controller
		dependRegister: function( dependType, handle ){
			dependHandle[ dependType ] = handle;
			jQuery.each( dependStack[ dependType ] || {}, function( depend, deferred ){
				dependHandleCall( dependType, depend, deferred );
			});
		},

		// jQuery.ready() --> call a function on dependencies ready
		ready: function( depends, callback ){
			var type = jQuery.type(depends);

			return typeExp.test(type) ?
				dependControl( depends, callback, 'ready', ++readyCount ) :
				type === 'function' ?
					DOMReayWhen.done(depends) :
					ready(depends);
		},

		// jQuery.provide()
		provide: function( name, depends, callback ){
			// save depends for debug
			dependStack._script[ name ] = depends;

			return dependControl( depends, callback, 'script', name );
		},

		// jQuery.plugin()
		plugin: function( name, depends, callback ){
			// save depends for debug
			dependStack._plugin[ name ] = depends;

			return dependControl( depends, callback, 'plugin', name );
		}
	});
})(jQuery);