/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.0
 * @info			http://www.jquerysdk.com
 *
 * @license			Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.event.special.mouse
 */

jQuery.plugin( 'jQuery.event.special.mouse', function( $, undefined ){

	var html = $('html'),
		
		getId = function( node, obj ){
			return ( node[ $.expando ] || (node[ $.expando ] = ++$.guid) )+'.'+( obj.guid || (obj.guid = ++$.guid) );
		},
	
		mousedown = function( originalBind, namespace, id ){
			return (mousedown[ id ] = function( event ){
					html.bind( 'mousemove.'+namespace, mousemove( originalBind, namespace, id ) )
						.bind( 'mouseup.'+namespace, mouseup( originalBind, namespace, id ) );

					mousemove[ id ].pageX = event.pageX;
					mousemove[ id ].pageY = event.pageY;
				});
		},
		
		mousemove = function( originalBind, namespace, id ){
			return (mousemove[ id ] = function( event ){

					event.type = originalBind.type;
					event.target = originalBind.originalTarget;
					event.data = originalBind.data;

					event.dragDeltaX = event.pageX - mousemove[ id ].pageX;
					event.dragDeltaY = event.pageY - mousemove[ id ].pageY;
					event.dragDelta  = delta  = Math.sqrt( Math.pow(event.dragDeltaX,2) + Math.pow(event.dragDeltaY,2) );

					mousemove[ id ].pageX = event.pageX;
					mousemove[ id ].pageY = event.pageY;

					return originalBind.originalHandler.apply(originalBind.originalTarget,arguments);
				});
		},
		
		mouseup = function( originalBind, namespace, id ){
			return (mouseup[ id ] = function(event){
			
					html.unbind( 'mousemove.'+namespace, mousemove[ id ] )
						.unbind( 'mouseup.'+namespace, mouseup[ id ] );

					delete( mousemove[ id ] );
					delete( mouseup[ id ] );
				});
		},
		
		mousewheel = function( originalBind, namespace, id ){
			return (mousewheel[ id ] = function( event ){
			
					var originalEvent = event.originalEvent;

					event.wheelDelta  = originalEvent.wheelDelta  ? originalEvent.wheelDelta  / 120 : originalEvent.detail ? originalEvent.detail*3 : 0;
					event.wheelDeltaX = originalEvent.wheelDeltaX ? originalEvent.wheelDeltaX / 120 : 0;
					event.wheelDeltaY = originalEvent.wheelDeltaY ? originalEvent.wheelDeltaY / 120 : event.wheelDelta;

					if( $.browser.opera ) {
						event.wheelDelta  *= -1;
						event.wheelDeltaX *= -1;
						event.wheelDeltaY *= -1;
					}

					event.type = originalBind.type;

					return originalBind.originalHandler.apply(this,arguments);
				});
		};

	$.extend( $.event.special, {
		mousedrag:{
			add: function( bind ) {
				bind.originalTarget = this;
				bind.originalHandler = bind.handler;
				
				var namespace = '_specialMousedrag';

				$(this).bind( 'mousedown.'+namespace, mousedown( bind, namespace, getId(this,bind) ) );
			},

			remove: function( unbind ) {
				var id = getId(this,unbind),
					namespace = '_specialMousedrag',
					handlerDown = mousedown[ id ],
					handlerMove = mousemove[ id ];
				
				$(this).unbind( 'mousedown.'+namespace, handlerDown );
				delete( handlerDown );
				
				html.unbind( 'mousemove.'+namespace, handlerMove );
				delete( handlerMove );
			}
		},
		mousewheel:{
			add: function( bind ) {
				bind.originalTarget = this;
				bind.originalHandler = bind.handler;
				
				var namespace = '_specialMousewheel',
					handler = mousewheel( bind, namespace, getId(this,bind) );

				if( $.browser.mozilla )
					$(this).bind( 'DOMMouseScroll.'+namespace, handler, false );
				else
					bind.handler = handler;
			},

			remove: function( unbind ) {
				var handler = mousewheel[ getId(this,unbind) ];
				
				if( $.browser.mozilla )
					$(this).unbind( 'DOMMouseScroll._specialMousewheel', handler, false );
				delete( handler );
			}
		}
	});

});