use strict;

use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::MeinPaket::API::XML::ResponseHandler;

use Readonly;
use Test::More;

# directory to MeinPaket.de sample files.
Readonly my $XMLSAMPLEDIR => $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/MeinPaket/t/Samples/';

sub Main {
    testDownloadCategories();
    testQueryOrders();
    done_testing();
}

sub testDownloadCategories {
    my $self = shift;
    my ($x) = @_;

    my $Result
      = DE_EPAGES::MeinPaket::API::XML::ResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'mp_example_20_09_downloadResponse_categories.xml', {'encoding' => 'utf-8'})});
    isa_ok( $Result, 'DE_EPAGES::MeinPaket::API::DTO::DownloadResponse' );
    is(scalar keys %{$Result->CategoryHash()}, 3238, '3238 categories' );

    my $Business = $Result->getCategory('1.1.21');
    isa_ok($Business, 'DE_EPAGES::MeinPaket::API::DTO::Category');
    is($Business->code(), '1.1.21', '1.1.21 category code');
    is($Business->parentCode(), '1.1','1.1.21 category parent');
    is($Business->name(), 'Business & Brse', '1.1.21 category name');

    my $Boerse = $Result->getCategory('1.1.21.2');
    isa_ok( $Boerse, 'DE_EPAGES::MeinPaket::API::DTO::Category' );
    is($Boerse->code(), '1.1.21.2', '1.1.21.2 category code');
    is($Boerse->parentCode(), '1.1.21','1.1.21.2 category parent');
    is($Boerse->name(), 'Brse & Aktien', '1.1.21.2 category name');

    # removed: category attributes not active anymore..?
    #    my $Hoerbuecher = $Result->getCategory('1.4');
    #    isa_ok($Hoerbuecher, 'DE_EPAGES::MeinPaket::API::DTO::Category');
    #    my $hAttributes = $Hoerbuecher->CategoryAttributeHash();
    #    is(ref $hAttributes, 'HASH', 'category attribute hash');
    #    is(scalar values %{$hAttributes}, 0, 'category attribute size');
    #    my $PublisherAttribute = $Hoerbuecher->getCategoryAttribute('Herausgeber');
    #    isa_ok($PublisherAttribute, 'DE_EPAGES::MeinPaket::API::DTO::CategoryAttribute');
    #    is($PublisherAttribute->code(),'Herausgeber','category attribute code');
    #    is($PublisherAttribute->search(),'true','category attribute search');
    #    is($PublisherAttribute->facet(),'false','category attribute facet');
    #    is($PublisherAttribute->name(),'Herausgeber','category attribute name');
    #    is($PublisherAttribute->description(), 'Herausgeber (xPlace)', 'category attribute description');

}

sub testQueryOrders {
    my $Result
      = DE_EPAGES::MeinPaket::API::XML::ResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'mp_example_30_02_order_response.xml', {'encoding' => 'utf-8'})});
    isa_ok($Result, 'DE_EPAGES::MeinPaket::API::DTO::QueryResponse');
    is(scalar @{$Result->OrderList()}, 1, 'one order');
    my $Order = $Result->popOrder();
    isa_ok($Order, 'DE_EPAGES::MeinPaket::API::DTO::Order');
    is($Order->currency(), 'EUR', 'currency');
    is($Order->deliveryMethod(), 'STANDARD', 'delivery method');
    is($Order->lastModificationDate(), '2010-10-26T19:52:09.163+02:00', 'last modification date');
    is($Order->orderId(), 'testOrders', 'order id');
    is($Order->totalDeliveryCosts(), '5.9', 'total delivery costs');
    is($Order->totalPrice(), '45.8', 'total price');
    my $BillingAddress = $Order->billingAddress();
    is($BillingAddress->city(), 'Frankfurt', 'billing: city');
    is($BillingAddress->company(), 'MaxMuster GmbH', 'billing: company');
    is($BillingAddress->country(), 'de', 'billing: country');
    is($BillingAddress->firstName(), 'Max', 'billing: first name');
    is($BillingAddress->houseNumber(), '2', 'billing: house number');
    is($BillingAddress->lastName(), 'Muster', 'billing: last name');
    is($BillingAddress->salutation(), 'Herr', 'billing: salutation');
    is($BillingAddress->street(), 'Musterstrasse', 'billing: street');
    is($BillingAddress->zipCode(), '69222', 'billing: zipcode');

    my $DeliveryAddress = $Order->deliveryAddress();
    is($DeliveryAddress->city(), 'Frankfurt', 'shipping: city');
    is($DeliveryAddress->country(), 'de', 'shipping: country');
    is($DeliveryAddress->firstName(), 'Max', 'shipping: first name');
    is($DeliveryAddress->houseNumber(), '1', 'shipping: house number');
    is($DeliveryAddress->lastName(), 'Muster', 'shipping: last name');
    is($DeliveryAddress->salutation(), 'Herr', 'shipping: salutation');
    is($DeliveryAddress->title(), 'Dr.', 'shipping: title');
    is($DeliveryAddress->street(), 'Musterstrasse', 'shipping: street');
    is($DeliveryAddress->zipCode(), '69222', 'shipping: zipcode');
}

RunScript( Sub => \&Main );

