use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite );
use DE_EPAGES::MeinPaket::API::Object::MeinPaketCategoryTree;
use DE_EPAGES::MeinPaket::API::Table::MeinPaketCategory qw ( GetInfoMeinPaketCategoriesByMeinPaketSiteID );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use Test::More;

sub testLocalCategoryTree {

    my $Tree;
    my $SiteID;
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            $Tree = DE_EPAGES::MeinPaket::API::Object::MeinPaketCategoryTree->new(
            'MeinPaketSite' => LoadObjectByPath('/Portals/MeinPaket/MeinPaket_de_DE_Sandbox'),
            'IsSandbox' => 1 );
            $SiteID = $Tree->siteID();
        }
    );

    my $ahInfoFromTable;
    RunOnStore(
        'Store' => 'MeinPaketCategories',
        'Sub'   => sub {
            $ahInfoFromTable = GetInfoMeinPaketCategoriesByMeinPaketSiteID($SiteID);
        }
    );

    plan skip_all => 'no categories in local Store!' unless (scalar @{$ahInfoFromTable});

    ok($SiteID > 0  , 'SiteID: '.$SiteID);

    my $TreeResult = $Tree->getLocalCategoryList();
    isa_ok($TreeResult,'DE_EPAGES::MeinPaket::API::DTO::DownloadResponse');

    is(scalar values %{$TreeResult->CategoryHash()}, scalar @{$ahInfoFromTable}, 'number of entries identical');

    foreach my $hInfo (@$ahInfoFromTable){
        my $Category = $TreeResult->getCategory($hInfo->{'ExternalCategoryID'});
        isa_ok($Category, 'DE_EPAGES::MeinPaket::API::DTO::Category', 'Category ' . $hInfo->{'ExternalCategoryID'});
        is($Category->parentCode()//'undef', $hInfo->{'ParentCategoryID'}//'undef','parent code is '. $hInfo->{'ParentCategoryID'}//'undef');
        if ($Category->parentCode()) {
            isa_ok($Category->parent(),'DE_EPAGES::MeinPaket::API::DTO::Category', 'Category::parent with code '.$Category->parentCode());
        }
    }

    my $aRootCategoriesFromResult = [ grep { $_->level() eq '1' } map { $_->{'parent'} = undef; $_ } values %{ $TreeResult->CategoryHash() } ];
    my $aRootCategories = $Tree->getRootCategories();
    is (scalar @{$aRootCategoriesFromResult} , scalar @{$aRootCategories}, 'root category list');

    foreach my $Category (@{$aRootCategories}){
        my $Match = $TreeResult->getCategory($Category->code());
        is($Category->name(), $Match->name(), 'root category name');
        is($Category->level(), $Match->level(), 'root category level');
    }
}

sub Main {
    testLocalCategoryTree();
    done_testing();
}

RunScript( Sub => \&Main );

1;
