use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MeinPaket::API::Object::RequestAdapter;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use Test::More;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            ## no critic (ProhibitMagicNumbers)

            # tests
            my $g = LoadObjectByPath('/Units/gram');
            my $kg = LoadObjectByPath('/Units/gram/kilogram');
            my $t  = LoadObjectByPath('/Units/gram/tone');

            # my $Price = 12.00;
            ok(!defined _RefPrice(12.00, undef, $kg), 'no RefContentAmount') or diag(_RefPrice(12.00, undef, $kg));
            ok(!defined _RefPrice(undef, 100, $kg), 'no price') or diag( _RefPrice(undef, 100, $kg));
            cmp_ok(_RefPrice(12.00, 1, $kg)->price(), '==', 12.00, '12.00 / kg');
            cmp_ok(_RefPrice(12.00, 10, $kg)->price(), '==', 1.20, '1.20 / kg');
            cmp_ok(_RefPrice(0.10, 100, $g)->price(), '==', 0.10, '0.10 / 100g');
            cmp_ok(_RefPrice(0.10, 10, $g)->price(), '==', 1.00, '1.00 / 100g');
            cmp_ok(_RefPrice(1200.00, 6, $t)->price(), '==', 0.20, '0.20 / kg');
            is(_RefPrice(1200.00, 6, $t)->toXML(), '<unitprice unit="per_1kg">0.20</unitprice>', 'serialization.');

            done_testing();
        }
    );
}

sub _RefPrice {
    my ($Price, $RefContentAmount, $OrderUnit) = @_;
    return DE_EPAGES::MeinPaket::API::Object::RequestAdapter::_unitPrice($Price, $RefContentAmount, $OrderUnit); ## no critic (ProtectPrivateSubs)
}

RunScript( Sub => \&Main );

1;