use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::WebInterface::API::ApplicationServer qw ( GetAppServer );
use Readonly;
use Test::More;

use DE_EPAGES::WebInterface::API::MessageCenter qw ( GetMessageCenterForScripts );



### mockup AppServer ####
no warnings 'redefine';    ## no critic (ProhibitNoWarnings)
sub DE_EPAGES::WebInterface::API::ApplicationServer::GetAppServer() { return MockAppServer->new(); }

Readonly my $ROOTPATH => '/Test';

## no critic (ProhibitMultiplePackages)

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                TestServletContext();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub TestServletContext {
    my $SubShop   = ExistsObjectByPath('/Test/TestSubShop') ? LoadObjectByPath('/Test/TestSubShop') : 
    LoadObjectByPath($ROOTPATH)->insertChild('Shop',{'Alias' => 'TestSubShop'});
    my $AppServer = DE_EPAGES::WebInterface::API::ApplicationServer::GetAppServer();
    isa_ok( $AppServer, 'MockAppServer', 'AppServer' );
    my $Servlet = $AppServer->currentServlet();
    ok( defined $Servlet, 'CurrentServlet' );
    my $Site = $Servlet->site();
    is( $Site->get('GUID'), $SubShop->get('GUID'), 'CurrentServlet site GUID match' );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'MasterShopEmpty.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

#---------------------------------------------------------------------------
#  MockAppServer
#---------------------------------------------------------------------------
package MockAppServer;

use base qw (DE_EPAGES::WebInterface::API::ApplicationServer);

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless \%args, $class;
    return $self;
}

sub currentServlet {
    return MockServlet->new();
}

sub messageCenter {
    return GetMessageCenterForScripts();
}

#---------------------------------------------------------------------------
#  MockServlet
#---------------------------------------------------------------------------
package MockServlet;

use DE_EPAGES::Object::API::Factory qw (
  LoadObjectByPath
  ExistsObjectByPath
);

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless \%args, $class;
    return $self;
}

sub site {
    return ExistsObjectByPath('/Test/TestSubShop') ? LoadObjectByPath('/Test/TestSubShop') : undef;
}

sub object {
    my $self = shift;
    return $self->{'Object'};
}
1;
