#!perl -w
use strict;
use Test::More tests => 49;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Object::XMLStructure;

## no critic qw( ProhibitLongLines )
RunScript(Sub => sub {

    my $hTest = {};
    my @Test;

    my $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_translation');

    ok( $XMLStruct->getXMLType() eq 'epages_translation', 'getXMLType 01');

    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');

    ok( $XMLStruct->getXMLType() eq 'epages_standard', 'getXMLType 02');

    #once with unknown XML-structure type
    #$XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('anything_anything');
    #ok( error , 'getXMLType 03');

    #existsElement
    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');
    ok( (not $XMLStruct->existsElement('anything')), 'existsElement 01');

    #addElement
    $XMLStruct->addElement('/KingKong', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object'}, {});
    ok( ($XMLStruct->existsElement('/KingKong')), 'existsElement 02, addElement 01');
    $XMLStruct->deleteElement('/KingKong');
    ok( not ($XMLStruct->existsElement('/KingKong')), 'deleteElement() 01');

    #test preparation
    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');
    $XMLStruct->addElement('/KingKong', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 0}, {});
    $XMLStruct->addElement('/KingKong/Sohn1', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 1}, {});
    $XMLStruct->addElement('/KingKong/Sohn2', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 2}, {});
    $XMLStruct->addElement('/KingKong1', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 3}, {});
    $XMLStruct->addElement('/KingKong1/Sohn1', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 4}, {});
    $XMLStruct->addElement('/KingKong1/Sohn2', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 5}, {});

    #testing
    @Test = ('/KingKong', '/KingKong1');
    is_deeply(($XMLStruct->getElementsWithoutParent()), \@Test, 'getElementsWithoutParent() 01');
    is_deeply(($XMLStruct->getRootElements()), \@Test, 'getRootElements() 01');
    @Test = ('/KingKong/Sohn1', '/KingKong/Sohn2');
    is_deeply(($XMLStruct->getChildElements('/KingKong')), \@Test, 'getChildElements() 01');
    @Test = ('/KingKong1/Sohn1', '/KingKong1/Sohn2');
    is_deeply(($XMLStruct->getChildElements('/KingKong1')), \@Test, 'getChildElements() 02');
    @Test = ('/KingKong', '/KingKong/Sohn1','/KingKong/Sohn2', '/KingKong1', '/KingKong1/Sohn1', '/KingKong1/Sohn2');
    is_deeply(($XMLStruct->getElements()), \@Test, 'getElements() 01');
    ok(($XMLStruct->existsParentElement('/KingKong/Sohn1') and $XMLStruct->existsElement('/KingKong/Sohn1')), 'existsParentElement() 01');
    #delte '/KingKong1'
    $XMLStruct->deleteElement('/KingKong1');
    ok((not $XMLStruct->existsParentElement('/KingKong1/Sohn1') and $XMLStruct->existsElement('/KingKong1/Sohn1')), 'existsParentElement() 02');
    #reverse delete '/KingKong1'
    $XMLStruct->addElement('/KingKong1', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 3}, {});
    is($XMLStruct->parentElement('/KingKong/Sohn1'), '/KingKong', 'parent() 01');
    isnt($XMLStruct->parentElement('/KingKong1/Sohn1'), '/KingKong', 'parent() 02');
    is($XMLStruct->parentElement('/KingKong1/Sohn1'), '/KingKong1', 'parent() 03');
    $XMLStruct->addElement('/GodZilla', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 6}, {});
    ok(($XMLStruct->existsSubElement('/KingKong')), 'existsSubElement() 01');
    ok(not ($XMLStruct->existsSubElement('/GodZilla')), 'existsSubElement() 02');

    #test preparation ...
    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');
    $XMLStruct->addElement('/KingKong', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 0}, {'height' => '25 feet', 'weight' => '60 tons', 'color' => 'black'});

    #testing
    ok(($XMLStruct->existsTagAttribute('/KingKong','height') and $XMLStruct->existsTagAttribute('/KingKong','weight') and $XMLStruct->existsTagAttribute('/KingKong','color')), 'existsTagAttribute() 01');
    @Test = ('height', 'weight', 'color');
    ok(not ($XMLStruct->existsAdditionalTagAttributes('/KingKong', \@Test )), 'existsAdditionalTagAttributes() 01');
    @Test = ('height', 'weight');
    ok(($XMLStruct->existsAdditionalTagAttributes('/KingKong', \@Test )), 'existsAdditionalTagAttributes() 02');
    @Test = ('color','height','weight');
    is_deeply($XMLStruct->getTagAttributes('/KingKong'), \@Test, 'getTagAttributes() 01');
    $hTest = {'height' => '25 feet', 'weight' => '60 tons', 'color' => 'black'};
    is_deeply($XMLStruct->getTagAttributesHash('/KingKong'), $hTest, 'getTagAttributesHash() 01');
    $XMLStruct->setTagAttributes('/KingKong', {'phone' => '03641-000000', 'color' => 'brown'});
    $hTest = {'height' => '25 feet', 'weight' => '60 tons', 'color' => 'brown','phone' => '03641-000000'};
    is_deeply($XMLStruct->getTagAttributesHash('/KingKong'), $hTest, 'setTagAttributesHash() 01');
    is($XMLStruct->countTagAttributes('/KingKong'), 4, 'countTagAttributes() 01');
    isnt($XMLStruct->countTagAttributes('/KingKong'), 6, 'countTagAttributes() 02');
    is($XMLStruct->getTagAttribute('/KingKong','color'), 'brown', 'getTagAttribute() 01');
    isnt($XMLStruct->getTagAttribute('/KingKong','color'), 'black', 'getTagAttribute() 02');
    $XMLStruct->setTagAttribute('/KingKong', 'color', 'black');
    is($XMLStruct->getTagAttribute('/KingKong','color'), 'black', 'setTagAttribute() 01');
    $XMLStruct->deleteTagAttribute('/KingKong', 'phone');
    ok(not ($XMLStruct->existsTagAttribute('/KingKong', 'phone')), 'deleteTagAttribute() 01');

    #test preparation
    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');
    $XMLStruct->addElement('/KingKong', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 0, 'AdditionalInfo' => 'nonsense'}, {});

    #testing
    ok(($XMLStruct->existsXMLStructAttribute('/KingKong', 'AdditionalInfo')), 'existsXMLStructAttribute() 01');
    is(($XMLStruct->getXMLStructAttribute('/KingKong', 'AdditionalInfo')), 'nonsense', 'getXMLStructAttribute() 01');
    $XMLStruct->setXMLStructAttribute('/KingKong', 'AdditionalInfo', 'kokolorus');
    is(($XMLStruct->getXMLStructAttribute('/KingKong', 'AdditionalInfo')), 'kokolorus', 'setXMLStructAttribute() 01');
    $XMLStruct->deleteXMLStructAttribute('/KingKong', 'AdditionalInfo');
    ok( not ($XMLStruct->existsXMLStructAttribute('/KingKong', 'AdditionalInfo')), 'deleteXMLStructAttribute() 01');
    $hTest = {'Identifier' => {'Alias' => 'KingKong'},'Class' => 'Object', 'TagName' => 'Object', 'Position' => 0};
    is_deeply($XMLStruct->getXMLStructAttributesHash('/KingKong'), $hTest, 'getXMLStructAttributesHash() 01');
    @Test = ('Class', 'Identifier', 'Position', 'TagName');
    is_deeply($XMLStruct->getXMLStructAttributes('/KingKong'), \@Test, 'getXMLStructAttributes() 01');

    #test preparation setXMLStructAttributes
    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');
    $XMLStruct->addElement('/KingKong', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 0, 'AdditionalInfo' => 'nonsense'}, {});

    #testing
    $hTest = {'Identifier' => {'Alias' => 'KingKong'}, 'Class' => 'Object', 'TagName' => 'Object', 'Position' => 0, 'AdditionalInfo' => 'nonsense'};
    is_deeply($XMLStruct->getXMLStructAttributesHash('/KingKong'), $hTest, 'getXMLStructAttributesHash() 01');
    $XMLStruct->setXMLStructAttributes('/KingKong', {'Class' => 'Object1', 'TagName' => 'UrObject', 'Position' => 999, 'AdditionalInfo' => 'kokolorus'});
    $hTest = {'Identifier' => {'Alias' => 'KingKong'}, 'Class' => 'Object1', 'TagName' => 'UrObject', 'Position' => 999, 'AdditionalInfo' => 'kokolorus'};
    is_deeply($XMLStruct->getXMLStructAttributesHash('/KingKong'), $hTest, 'setXMLStructAttributes() 01');

    #test preparation
    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');
    $XMLStruct->addElement('epages', {}, {'Class' => 'epages', 'TagName' => 'epages', 'Position' => 0, 'NonEffectiveForSubPathes' => '1'}, {});
    $XMLStruct->addElement('/KingKong', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 1}, {});
    $XMLStruct->addElement('/KingKong/Sohn1', {'Alias' => 'Sohn1'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 2}, {});
    $XMLStruct->addElement('/KingKong/Sohn2', {'Alias' => 'Sohn2'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 2}, {});

    #testing
    ok(not ($XMLStruct->isEmpty()), 'isEmpty() 01');
    $XMLStruct->setXMLStructAttribute('/KingKong', 'NonEffectiveForSubPathes', 1);
    $XMLStruct->setXMLStructAttribute('/KingKong/Sohn1', 'NonEffectiveForSubPathes', 1);
    $XMLStruct->setXMLStructAttribute('/KingKong/Sohn2', 'NonEffectiveForSubPathes', 1);
    ok(($XMLStruct->isEmpty()), 'isEmpty() 02');

    #test preparation
    $XMLStruct = DE_EPAGES::Object::API::Object::XMLStructure->new('epages_standard');
    $XMLStruct->addElement('/KingKong', {'Alias' => 'KingKong'}, {'Class' => 'Object', 'TagName' => 'Object', 'Position' => 0}, {'reference' => 1});
    $XMLStruct->addElement('/KingKong/RoleAction{Action->Action1;Class->Class1}', {'Action' => 'Action1', 'Class' => 'Class1'}, {'Class' => 'RoleAction', 'TagName' => 'RoleAction', 'Position' => 1}, {'reference' => 1});

    $hTest = {'Alias' => 'KingKong'};
    is_deeply($XMLStruct->getID('/KingKong'), $hTest, 'getID() 01');
    $hTest = {'Action' => 'Action1', 'Class' => 'Class1'};
    is_deeply($XMLStruct->getID('/KingKong/RoleAction{Action->Action1;Class->Class1}'), $hTest, 'getID() 02');
    ok(($XMLStruct->existsIDPart('/KingKong','Alias')), 'existsIDPart() 01');
    ok(not ($XMLStruct->existsIDPart('/KingKong','Anything')), 'existsIDPart() 02');
    ok(
        (
            $XMLStruct->existsIDPart('/KingKong/RoleAction{Action->Action1;Class->Class1}','Action') and
            $XMLStruct->existsIDPart('/KingKong/RoleAction{Action->Action1;Class->Class1}','Class') and not
            $XMLStruct->existsIDPart('/KingKong/RoleAction{Action->Action1;Class->Class1}','Anything')
        ),
        'existsIDPart() 03'
    );
    is($XMLStruct->getIDValue('/KingKong/RoleAction{Action->Action1;Class->Class1}', 'Action'), 'Action1', 'getIDValue() 01');
    is($XMLStruct->getIDValue('/Anything', 'Action'), undef, 'getIDValue() 03');
    is($XMLStruct->getIDValue('/KingKong/RoleAction{Action->Action1;Class->Class1}', 'Anything'), 0, 'getIDValue() 03');

    is($XMLStruct->getClass('/KingKong/RoleAction{Action->Action1;Class->Class1}'), 'RoleAction', 'getClass() 01');

    $XMLStruct->setID('/KingKong', {'Anything' => 'Anything'});
    $hTest = {'Anything' => 'Anything'};
    is_deeply($XMLStruct->getID('/KingKong'), $hTest, 'setID() 01');


    #getXMLTypeAddictedID

    #isRootElement
    #getDirectChildElements
    #assimilateXMLStructure
    #deleteChilds

}, Debug=>1);

1;
