use Test::More tests => 4;
use warnings;
use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject ExistsObjectByPath );
use DE_EPAGES::Presentation::API::SiteTLEProcessor qw(ReplaceTLEs);

sub ProcessorTest {

    my $Object  = LoadObjectByPath('/Test');

    # test if the SiteTLEProcessor is using the correct context
    # access to parent objects is not allowed

    ok( ReplaceTLEs('#Alias',$Object->tleHash) eq 'Test', 'replace TLE #Alias -> Test' );
    ok( ReplaceTLEs('#Parent',$Object->tleHash) eq '#Parent', 'replace TLE #Parent -> #Parent' );
    ok( ReplaceTLEs('#Parent.Alias',$Object->tleHash) eq '#Parent.Alias', 'replace TLE #Parent.Alias -> #Parent.Alias' );
    ok( ReplaceTLEs('#System',$Object->tleHash) eq '#System', 'replace TLE #System -> #System' );
}


sub Cleanup {
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        Init();
        ProcessorTest();
        Cleanup();
    });
});
