use strict;
use Test::More tests => 8;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle );
use DE_EPAGES::ShopTransfer::API::Import;

## no critic qw( ProhibitMagicNumbers )
sub Main {
    ConnectTestSite();

    my $dbi = GetCurrentDBHandle();
    my $driverName = $dbi->driverName;
    ok( $dbi, 'GetCurrentDBHandle driverName='.$driverName );

    ok( Init( $dbi ), 'Init' );

    SKIP: {
        skip( 'test to import empty string only neccesary for mysql', 5 ) if $driverName ne 'mysql';

        my $import = DE_EPAGES::ShopTransfer::API::Import->new();

        ok( $import->importTable('importest', 'ImporTestShop', {storePath=>'.'})==0, 'import test table');

        my $sql = 'SELECT id, name_notnull, name_null FROM importest ORDER BY id';
        my $aaRows = $dbi->execute($sql, [], [qw(Integer String String)], [qw(ID NameNotNull NameNull)]);

        ok( $aaRows->[0]->{'NameNull'} eq 'string2', 'simple string' );
        ok( ! defined $aaRows->[1]->{'NameNull'} , 'undef (NULL)' );
        ok( $aaRows->[2]->{'NameNotNull'} eq '', 'empty string' );
        ok( $aaRows->[3]->{'NameNotNull'} eq ' ', 'space character' );
    }

    ok( Cleanup( $dbi ), 'Cleanup' );

    DisconnectSite();
}

sub Init {
    my ( $dbi ) = @_;
    Cleanup( $dbi );

    $dbi->do(<<END_SQL);
CREATE TABLE importest(
    id INTEGER,
    name_notnull VARCHAR(50) NOT NULL,
    name_null VARCHAR(50) NULL
)
END_SQL

    return $dbi->existsTableColumn( 'importest', 'name_notnull' );
}

sub Cleanup {
    my ( $dbi ) = @_;

    $dbi->dropTableIfExists( 'importest' );

    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
