use strict;
use Config;
use Encode qw (encode);
use Test::Simple tests => 5;
use Readonly;
use DE_EPAGES::Core::API::File qw (DeleteFile ExistsFile WriteFile);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::SipsAtos::API::Constants qw (GetBinDirectory TEST_MERCHANT_ID);

Readonly::Scalar my $TRANSACTION_OFFSET => 6;
my $pathfile = "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/SipsAtos/Data/param/pathfile";

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my %Params = (
        'merchant_id'      => TEST_MERCHANT_ID,
        'merchant_country' => 'fr',
        'amount'           => '1230', # 12.30
        'currency_code'    => '978',  # EUR
        'capture_mode'     => 'VALIDATION',
        'capture_day'      => '6',
        'language'         => 'en',
        'pathfile'         => $pathfile,
        'transaction_id'   => substr(time, -$TRANSACTION_OFFSET , $TRANSACTION_OFFSET ),
        'bgcolor'          => 'FFFFFF',
        'textcolor'        => '000000',
    );

    my $path_bin = GetBinDirectory() . 'request';
    my $parm = join(' ', map { qq($_="$Params{$_}") } keys %Params);
    $parm = encode('iso-8859-1', $parm);

    ##no critic(ProhibitTwoArgOpen)
    ok(open(my $INFO, "$path_bin $parm|"), 'call request binary');
    my $result = do { local $/ = undef; <$INFO> };
    ok(close($INFO), 'closing');
    ##use critic

    my (undef, $code, $error, $message) = split(/!/, $result);
    $code    = '' unless defined $code;
    $error   = '' unless defined $error;
    $message = '' unless defined $message;
    ok($code ne '' && $code == 0, 'request binary result');

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    DeleteFile($pathfile) if (ExistsFile($pathfile));
    return 1;
}

sub Init {
    Cleanup();
    WriteFile($pathfile, _pathfileContent());
    return 1;
}

sub _pathfileContent {
    shift;

    my $dir = LoadRootObject()->get('WebRoot');
    my $Path = "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/SipsAtos/Data/param";
    if ($Config{'osname'} eq 'MSWin32') {
        $Path =~ s/\//\\/g;
        $Path .= '\\';
    }
    else {
        $Path .= '/';
    }

    my (@Lines) = ();
    push(@Lines, 'DEBUG!NO!');
    push(@Lines, "D_LOGO!$dir/SF/SipsAtos/logo/!");
    push(@Lines, "F_DEFAULT!$Path" . 'parmcom.defaut!');
    push(@Lines, "F_PARAM!$Path" . 'parmcom!');
    push(@Lines, "F_CERTIFICATE!$Path" . 'certif!');

    return \join("\n", @Lines);
}

RunScript(Sub=>\&Main, Debug=>1);
