#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::File  qw ( GetFileContent WriteFile DeleteFile ExistsFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::TLE::API::Template;
use DE_EPAGES::TLE::t::tlevars;

our $TEST_ONE;

sub Main {
    Initialize();

    my $Template = DE_EPAGES::TLE::API::Template->new;

    if ( defined $TEST_ONE ) {
        plan tests => 1;
        ok ( Test_Package($Template, $TEST_ONE ), $TEST_ONE );
    }
    else {
        my @Files = glob '*_t.tmpl';
        plan tests => 2 + 3 * (scalar @Files);

        Test_Files( $Template, @Files );

        ok( ExistsFile( "$_.ctmpl" ), "compiled template exists: $_.ctmpl" ) foreach @Files;

        # EPG-16370: test overloded Template that creates no ctmpl files
        DeleteFile( $_ ) foreach glob '*.ctmpl';
        my @Ctmpls = glob '*.ctmpl';
        ok( 0 == scalar @Ctmpls, 'all ctmpl files deleted' );

        my $NoCtmplTemplate = NoCtmplTemplate->new;
        Test_Files( $NoCtmplTemplate, @Files );

        @Ctmpls = glob '*.ctmpl';
        ok( 0 == scalar @Ctmpls, 'no ctmpl files created' );
    }
}

local $| = 1;
RunScript( Sub => sub {
    Main(@ARGV);
});

if (ExistsError()) {
    print GetError()->shortMessage(),"\n";
    exit 1;
} else {
    print ".\n";
}

sub ErrorDefinition {
    return {
        NO_EQUAL_LINE => 'Output is different at line #AtLine'
    };
}

sub Test_Files {
    my ($Template, @Files) = @_;

    foreach my $File ( @Files ) {
        if ($File =~ /^(.+)_t\.tmpl$/) {
            my $TestCase = $1;
            ok ( Test_Package( $Template, $TestCase ), $File );
        }
        else {
            die "invalid file name $File";
        }
    }
}

sub Test_Package {
    my ($Template, $i) = @_;

    my $isContent;
    eval {
        my $Loops = DE_EPAGES::TLE::t::tlevars::GetLoops($i);
        my $Vars  = DE_EPAGES::TLE::t::tlevars::GetVars($i);

        $isContent = $Template->process($i.'_t.tmpl', { %$Vars , %$Loops});
        my $rExContent = GetFileContent($i.'_e.tmpl');
        CompareContent($rExContent, \$isContent);
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->shortMessage();
        $Error->{'Vars'}->{'Test'} = $i;
        WriteFile($i.'_i.tmpl', \$isContent) if defined $isContent;
        return 0;
    } else {
        DeleteFile($i.'_i.tmpl');
    }
    return 1;
}

sub CompareContent {
    my ($rExpected, $rIs) = @_;

    my @aExpected = split /\n/, $$rExpected;
    my @aIs = split /\n/, $$rIs;
    if (scalar @aIs != scalar @aExpected) {
        Error('NO_EQUAL_LINES', {'CountLinesIs' => scalar @aIs, 'CountLinesExpected' => scalar @aExpected } );
    }
    for (my $i=0;$i < scalar @aExpected; $i++) {
        if ($aExpected[$i] ne $aIs[$i]) {
            Error('NO_EQUAL_LINE', {'AtLine' => $i + 1, 'Exp'=>$aExpected[$i], 'Ist'=> $aIs[$i] } );
        }
    }
    return 1;
}

sub Initialize {
}

sub Cleanup {
    DeleteFile( $_ ) foreach glob '*.ctmpl';
}

package NoCtmplTemplate;
use base 'DE_EPAGES::TLE::API::Template';

sub perlFileName {
    return undef;
}

1;
