IF OBJECT_ID('sp_ResetTaxMatrix') IS NOT NULL
BEGIN
    DROP PROCEDURE sp_ResetTaxMatrix
END
go

-- resets the tax matrix of a shop to the system tax matrix, use taxmatrixalias from diagnostics
CREATE PROCEDURE sp_ResetTaxMatrix( @ShopID int) AS
BEGIN

    DECLARE @TransactionCount int
    SELECT @TransactionCount = @@trancount

    IF @TransactionCount > 0 SAVE  TRANSACTION resettaxmatrix
    ELSE                     BEGIN TRANSACTION resettaxmatrix

        IF NOT EXISTS( SELECT 1 FROM shop WHERE shopid = @ShopID )
        BEGIN
            PRINT "Shop %1! not found", @ShopID
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN
        END

        -- find the current shop tax matrix
        DECLARE @ShopTaxMatrixAttributeID int
        SELECT @ShopTaxMatrixAttributeID = a.attributeid
        -- SELECT a.attributeid, ao.alias, co.alias
        FROM class c
        INNER JOIN attribute a ON a.attributeclassid = c.classid
        INNER JOIN object ao ON ao.objectid = a.attributeid
        INNER JOIN object co ON co.objectid = c.classid
        INNER JOIN object cpo ON cpo.objectid = co.parentid
        WHERE
          cpo.alias = 'Classes'
          AND cpo.parentid = 1
          AND co.alias = 'Shop'
          AND ao.alias = 'TaxMatrix'
    
        PRINT "ShopTaxMatrixAttributeID=%1!", @ShopTaxMatrixAttributeID
    
        DECLARE @ShopTaxMatrixID int
        SELECT @ShopTaxMatrixID = ra.referencedobjectid
        FROM referenceattribute ra
        WHERE ra.objectid = @ShopID
          AND ra.attributeid = @ShopTaxMatrixAttributeID
    
        PRINT "ShopTaxMatrixID=%1!", @ShopTaxMatrixID
    
        -- get the system tax matrix
        DECLARE @SystemTaxMatrixAttributeID int
        SELECT @SystemTaxMatrixAttributeID = a.attributeid
        -- SELECT a.attributeid, ao.alias, co.alias
        FROM class c
        INNER JOIN attribute a ON a.attributeclassid = c.classid
        INNER JOIN object ao ON ao.objectid = a.attributeid
        INNER JOIN object co ON co.objectid = c.classid
        INNER JOIN object cpo ON cpo.objectid = co.parentid
        WHERE
          cpo.alias = 'Classes'
          AND cpo.parentid = 1
          AND co.alias = 'System'
          AND ao.alias = 'TaxMatrix'
    
        PRINT "SystemTaxMatrixAttributeID=%1!", @SystemTaxMatrixAttributeID
    
        DECLARE @SystemTaxMatrixID int
        SELECT @SystemTaxMatrixID = ra.referencedobjectid
        FROM referenceattribute ra
        WHERE ra.objectid = 1
          AND ra.attributeid = @SystemTaxMatrixAttributeID

        PRINT "SystemTaxMatrixID=%1!", @SystemTaxMatrixID
        IF @SystemTaxMatrixID IS NULL
        BEGIN
            PRINT "System tax matrix not found"
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN
        END
    
        IF  @ShopTaxMatrixID is not null and @SystemTaxMatrixID <> @ShopTaxMatrixID
        BEGIN
            PRINT "Tax is already completely customized"
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN
        END
    
        SELECT shoptmo.objectid, CONVERT(VARCHAR(30), shoptmo.alias) AS ShopTaxMatrix
        FROM object shoptmo
        WHERE shoptmo.parentid = @ShopID
          AND shoptmo.alias LIKE 'TaxMatrix%'
    
        DECLARE @OldShopTaxMatrixID int
        SELECT @OldShopTaxMatrixID = shoptmo.objectid
        FROM object shoptmo
        WHERE shoptmo.parentid = @ShopID
          AND shoptmo.alias LIKE 'TaxMatrix%'
    
        SELECT objectid, CONVERT(VARCHAR(30),alias) AS "selected ShopTaxMatrix"
        FROM object WHERE objectid = @OldShopTaxMatrixID
    
        IF @OldShopTaxMatrixID IS NULL
        BEGIN
            PRINT "Old shop tax matrix not found (tax matrix has not been customized)"
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN
        END
        
        PRINT "%1! customers updated", @@rowcount
    
        UPDATE lineitemcontainer SET taxareaid = systa.taxareaid
        FROM lineitemcontainer lic
        INNER JOIN taxarea shopta ON shopta.taxareaid = lic.taxareaid
        INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shopta.taxmatrixid
        INNER JOIN object shoptao  ON shoptao.objectid = shopta.taxareaid
        INNER JOIN object systao   ON systao.alias = shoptao.alias
        INNER JOIN taxarea systa ON systa.taxareaid = systao.objectid
        WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
          AND systa.taxmatrixid = @SystemTaxMatrixID

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN 1
        END
    
        PRINT "%1! lineitemcountainers updated", @@rowcount
    
        UPDATE product SET taxclassid = systc.taxclassid
        FROM product p
        INNER JOIN taxclass shoptc ON shoptc.taxclassid = p.taxclassid
        INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
        INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
        INNER JOIN object systco   ON systco.alias = shoptco.alias
        INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
        WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
          AND systc.taxmatrixid = @SystemTaxMatrixID

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN 1
        END

        PRINT "%1! products updated", @@rowcount
    
        UPDATE lineitem SET taxclassid = systc.taxclassid
        FROM lineitem li
        INNER JOIN taxclass shoptc ON shoptc.taxclassid = li.taxclassid
        INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
        INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
        INNER JOIN object systco   ON systco.alias = shoptco.alias
        INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
        WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
          AND systc.taxmatrixid = @SystemTaxMatrixID

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN 1
        END
    
        PRINT "%1! lineitems updated", @@rowcount
    
        UPDATE paymentmethod SET taxclassid = systc.taxclassid
        FROM paymentmethod pm
        INNER JOIN taxclass shoptc ON shoptc.taxclassid = pm.taxclassid
        INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
        INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
        INNER JOIN object systco   ON systco.alias = shoptco.alias
        INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
        WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
          AND systc.taxmatrixid = @SystemTaxMatrixID

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN 1
        END
    
        PRINT "%1! paymentmethods updated", @@rowcount
    
        UPDATE shippingmethod SET taxclassid = systc.taxclassid
        FROM shippingmethod sm
        INNER JOIN taxclass shoptc ON shoptc.taxclassid = sm.taxclassid
        INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
        INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
        INNER JOIN object systco   ON systco.alias = shoptco.alias
        INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
        WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
          AND systc.taxmatrixid = @SystemTaxMatrixID

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN 1
        END

        PRINT "%1! shippingmethods updated", @@rowcount

        UPDATE shippingoption SET taxclassid = systc.taxclassid
        FROM shippingoption so
        INNER JOIN taxclass shoptc ON shoptc.taxclassid = so.taxclassid
        INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
        INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
        INNER JOIN object systco   ON systco.alias = shoptco.alias
        INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
        WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
          AND systc.taxmatrixid = @SystemTaxMatrixID

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN 1
        END

        PRINT "%1! shippingoptions updated", @@rowcount

        -- reset the shop tax matrix
        UPDATE referenceattribute
        SET referencedobjectid = @SystemTaxMatrixID
        WHERE objectid = @ShopID
          AND attributeid = @ShopTaxMatrixAttributeID

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION resettaxmatrix
            RETURN 1
        END

        PRINT "Delete (via deleteObject.pl) the old shoptaxmatrix %1!." , @OldShopTaxMatrixID

    IF @TransactionCount = 0
        COMMIT TRANSACTION resettaxmatrix

    RETURN 0
END
go
