use strict;
use Test::More tests => 4;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::TrustedRating::API::TrustedRating qw ( UpdateRatingState );

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval { UpdateRatingState( 'X41495A6E65ECDDCD554A02C0601D1C97', 0 ) };
            ok( !ExistsError(), 'TS-ID is ok. State has changed to inactive.' ) or diag( GetError()->longMessage() );
            eval { UpdateRatingState( 'X41495A6E65ECDDCD554A02C0601D1C97', 1 ) };
            ok( !ExistsError(), 'TS-ID is ok. State has changed to active.' ) or diag( GetError()->longMessage() );
            eval { UpdateRatingState( 'X41495A6E65ECDDCD554A02C0601D', 0 ) };
            ok( ExistsError && GetError()->code eq 'INVALID_TSID', 'TS-ID is invalid.' );
            eval { UpdateRatingState( '', 0 ) };
            ok( ExistsError && GetError()->code eq 'STRING_EMPTY', 'TS-ID is empty.' );
        }
    );
}

RunScript( Sub => \&Main, Debug => 1 );
