use 5.010;
use strict;
use warnings;
## no critic( RequireCorrectImports RequireTestLabels )
## no critic( RequireExistsError RequireExtendedFormatting) EP
use LWP::UserAgent;
use XML::LibXML;
use Getopt::Std;

use Test::More;
use Test::Exception;

use DE_EPAGES::Twenga::API::Impl;

use constant CTXML => 'text/xml; charset=UTF-8';

sub xml {
    my( $twenga, $rsp, $msg_rex ) = @_;
    $msg_rex ||= qr/^true$/;
    # alte Methode
    my $doc = eval { XML::LibXML->load_xml( string => $rsp->content ) };
    ok $doc, 'parsed content into document';
    my $xpc = XML::LibXML::XPathContext->new( $doc );
    my @nodes = $xpc->findnodes( '/response' );
    is scalar @nodes, 1;
    @nodes = $xpc->findnodes( '/response/message' );
    is scalar @nodes, 1;
    my $val = $xpc->findvalue( '/response/message' );
    like $val, $msg_rex, $msg_rex;
    # neue Methode
    my $hash = $twenga->data( $rsp );
    0 and diag explain $hash;
    like $hash->{content}, $msg_rex, $msg_rex;
    ok ! $hash->{error};
    ok ! $hash->{errormessage};
    return;
}

my $VRB;
my $mrcacc;
{
    getopts 'hvK:', \my %opt;
    if ( $opt{h} ) {
        print <<'EOS';
  -h            Hilfe
  -v            verbos
  -K            Konto
EOS
        exit 0;
    }
    $VRB = 1 if $opt{v};
    # Welches Händlerkonto benutzen?
    my %merchant_accounts = (
        tmetzinger  =>
    [qw[tmetzinger@epages.de   8uakpusr    aa541ef3422583424d4ad9bb4681b2f0 FR]],
        balu        =>
    [qw[balu@epages.de         n/oxklbb    0ffaca95e3e5242ba1097ad8a9a6e95d DE]],
        test        =>
    [qw[test@epages.de         @4hbnatv    2d2c8394e31101a261abf1784302bf75 GB]],
    );
    $opt{K} //= 'balu'; # Vorgabekonto
    $mrcacc = $merchant_accounts{ $opt{K} } or die "kein Konto [$opt{K}]!";
    diag "Konto: @$mrcacc";
}

# Partner

# prod_epages   D5vuV9Z98n
# dev_epages    sxwoxhfp

my $api_user = 'prod_epages';
my $api_pswd = 'D5vuV9Z98n';

my $twenga = Twenga->new(
    partner_auth_key => '1f2027e88b6964f3b952212b6d15397ae2a7d74a',
#   test_mode        => 1,
#   partner_username => $api_user,
#   partner_password => $api_pswd
);
isa_ok $twenga, 'Twenga';

# Der PARTNER_AUTH_KEY zeigt Twengas Planlosigkeit. Es handelt sich um eine
# MD5-Prüfsumme in hexadezimaler Notation. Diese kann per definitionem direkt
# in die URL gepackt werden. Twenga kodiert die Prüfsumme aber als Base64, als
# wären es binäre Daten, die über ein Textprotokoll (wie SMTP) übertragen
# werden müssen. So entstehen Gleichheitszeichen am Ende. Und nun wird deshalb
# auch noch URL-Kodierung fällig. Für dies desorientiert wirkende Vorgehen ist
# keinerlei Veranlassung zu entdecken.

is $twenga->partner_auth_key_b64,
    'MWYyMDI3ZTg4YjY5NjRmM2I5NTIyMTJiNmQxNTM5N2FlMmE3ZDc0YQ=='
    or die 'PARTNER_AUTH_KEY falsch';

my $api_url = $twenga->api_url;
is $api_url, 'https://rts.twenga.com/api/', 'API URL ok';

my $twenga_url = qr#^$api_url\w+/\w+/?\?.*PARTNER_AUTH_KEY#;

my $doc;
my $xpc;
my @nodes;
my $val;

# GetSubscriptionLink lieferte mit und ohne Twenga-HTTP-Authentifizierung
# einen Fehler:
#   <error><code>403</code><message>Authentication failed.</message></error>
# Dies lag am falschen PARTNER_AUTH_KEY-Parameter.

my $getsublink = $twenga->make_req( Site => GetSubscriptionLink => {
    country     => 'FR',
    feed_url    => 'http://bla.de/feed.txt',
    site_url    => 'http://bla.de/',
} );
isa_ok $getsublink, 'HTTP::Request';
like $getsublink->uri, $twenga_url, 'URL ~ ok';
$VRB and diag $getsublink->as_string;
ok ! $getsublink->header( 'Authorization' ), 'no Authorization header';

my $ua = LWP::UserAgent->new( keep_alive => 1 );
my $rsp = $ua->request( $getsublink );
ok $rsp->is_success, 'response success';
is $rsp->header('content-type'), CTXML;
$VRB and diag $rsp->as_string;

throws_ok { $twenga->make_req( Site => Exist => {} ) }
    qr/\bAttribute\b.*\bkey\b/;

xml $twenga, $rsp, qr#^https://rts.twenga.com/subscribe/\?subscription_key=\w+$#;

$twenga->merchant_login(    $mrcacc->[0] );
$twenga->merchant_password( $mrcacc->[1] );
$twenga->merchant_hashkey(  $mrcacc->[2] );

my $site_exist = $twenga->make_req( Site => Exist => { } );
isa_ok $site_exist, 'HTTP::Request';
like $site_exist->uri, $twenga_url, 'URL ~ ok';
my $auth = $site_exist->header( 'Authorization' );
ok $auth, 'has Authorization header';
is $auth, "Basic NTVkNjE1YTk1N2RjYzcwYTRiYzFhMTAwYmZjMDcwOTI6MTEyZjVhMWNlZjZmOThmNTMwYjY0ZjU0MDBhYmEwMzI=\n";
$VRB and diag $site_exist->as_string;

$rsp = $ua->request( $site_exist );
ok $rsp->is_success, 'response success';
is $rsp->header('content-type'), CTXML;
$VRB and diag $rsp->as_string;





my $site_act = $twenga->make_req( Site => Activate => { } );
ok $site_act->header( 'Authorization' );
$rsp = $ua->request( $site_exist );
ok $rsp->is_success, 'response success';
is $rsp->header('content-type'), CTXML;
$VRB and diag $rsp->as_string;

xml $twenga, $rsp;



my $site_addfeed = $twenga->make_req( Site => AddFeed => {
    feed_url => 'http://blubbelblub.de/gurkensalat',
} );
ok $site_addfeed->header( 'Authorization' );
$rsp = $ua->request( $site_exist );
ok $rsp->is_success, 'response success';
is $rsp->header('content-type'), CTXML;
$VRB and diag $rsp->as_string;

xml $twenga, $rsp;



my $site_getsublink = $twenga->make_req( Site => GetSubscriptionLink => {
    country     => 'DE',
    site_url    => 'http://www.gurkenbauer.de/',
    feed_url    => 'http://www.gurkenbauer.de/hastunichgesehen',
});



my $site_getscript = $twenga->make_req( Site => GetTrackingScript => {
    basket_id       => 99,
    order_id        => 3456789,
    cli_email       => 'mludwig@epages.de',
    total_ht        => 88.79,
    user_id         => 878787,
});
ok $site_getscript->header( 'Authorization' );
$rsp = $ua->request( $site_getscript );
ok $rsp->is_success, 'response success';
is $rsp->header('content-type'), CTXML;
$VRB and diag $rsp->as_string;

xml $twenga, $rsp, qr#^\s*<script language="javascript" type="text/javascript"#;
my $content = $rsp->decoded_content;
like   $content, qr#\bhttps://#, 'https found';
unlike $content, qr#\bhttp://#, 'no http present';

done_testing;
