#!/bin/env perl
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBN+iWSl5jMabHSgyARAhujAKDPDsTq8jpw3cqxKNQRwPR+8t1SxgCgrzBm
#1PLd2pyO4R5lnorFszyFxhw=
#=64LX
#-----END PGP SIGNATURE-----
#line 2
use strict;

use Getopt::Long;
use Config::IniFiles;
use Net::INET6Glue;
use Net::SMTP;
use Sys::Hostname;

sub Main {
    local $| = 1;

    my %Options;
    my @GetOpt = qw(ini=s section=s account=s from=s to=s subject=s file=s debug help);

    GetOptions(\%Options, @GetOpt) or usage();
    usage() if $Options{help};
    usage('Missing parameter -file') unless $Options{file};
    usage('Parameter -file with nonexisting file: '. $Options{file}) unless -f $Options{file};
    usage('Parameter -file with empty file: '. $Options{file}) unless -s $Options{file};

    my $HostName = hostname();
    chomp($HostName);
    $HostName =~ s/\..*//;

    my ($IniConfig, $MailServer, $MailLogin, $MailPasswd);
    if ( defined $Options{account} ) {
        $MailServer = $Options{account};
    }
    elsif ( $Options{ini} ) {
        $Options{section} = ':Default:' unless $Options{section};
        $IniConfig = Config::IniFiles->new(-file => $Options{ini}, -default => ':Default:') or die $!;
        $MailServer = $IniConfig->val( $Options{section}, "MailConnection:$HostName" );
        $MailServer = $IniConfig->val( $Options{section}, 'MailConnection' ) unless defined $MailServer;
    }

    if ($MailServer) {
        if( $MailServer =~ s/(.+)\@// ) {
            $MailLogin = $1;
        }
        if( $MailLogin =~ s/(.+):// ) {
            $MailPasswd = $1;
        }
    } else {
        $MailServer = 'localhost';
    }

    my ($From, $To) = ($Options{from}, $Options{to});
    if ( $Options{ini} and not $From ) {
        $From = $IniConfig->val( $Options{section}, "MailSender:$HostName" );
        $From = $IniConfig->val( $Options{section}, 'MailSender' ) unless defined $From;
    }
    if ( $Options{ini} and not $To ) {
        $To = $IniConfig->val( $Options{section}, "MailRecipient:$HostName" );
        $To = $IniConfig->val( $Options{section}, 'MailRecipient' ) unless defined $To;
    }

    $From = $MailLogin. '@'. $MailServer if !$From && $MailLogin;
    $From or $From = (getpwuid($<))[0]. '@'. $MailServer;
    $To = $From unless $From;

    my $MailHeader = "From: $From\nTo: $To\nSubject: $Options{subject}\n";

    open my $FOO, '<', $Options{file} or die "Can't open $Options{file}: $!";
    my $Content =  do { local $/ = undef; <$FOO> };
    close $FOO or die "Can't close $Options{file}: $!";

    my $Smtp = Net::SMTP->new($MailServer, Debug => $Options{debug});
    $Smtp->auth($MailLogin, $MailPasswd) if $MailLogin;
    foreach (split/[,;]/, $To) {
        $Smtp->mail($From); $Smtp->to($_);
        $Smtp->data(); $Smtp->datasend($MailHeader);
        $Smtp->datasend("$Content\n");
        $Smtp->dataend();
    }
    $Smtp->quit;
}

sub usage {
    my ($Message) = @_;

    print STDERR "$Message\n\n" if defined $Message;

    print <<END_USAGE;
Usage:
    perl $0 [Options] [Flags]

Options:
    -ini       ini file that contains section -section
    -section   value of key 'MailConnection' in section is used as mail server
                 default: :Default:
    -account   mail server account (passwd:user\@server; passwd and user opt.)
                 default: value of key 'MailConnection', else -from
    -from      sender (e-mail address), defaults:
                 1. value of key 'MailSender'
                 2. else: login of MailConnection
                 3. else: login\@host
    -to        list of recipients separated by [,;], defaults:
                 1. value of key 'MailRecipient'
                 2. else: -from
    -subject   e-mail subject (optional)
    -file      name of file to sent (mandatory)

Flags:
    -help        show the command line options

Example:
    perl $0 -to "ep_appl\@localhost" -subject "Output from SybaseBackup.sh" -file SybaseBackup.log
END_USAGE
    exit 2;
}

Main();

1;
