#!/bin/bash

NEW_VER=6.12.0

BN=`basename $0`
DN=`cd $(dirname $0) && pwd`

umask 022

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

VARIABLES_SOURCE=environment

if [[ -z $EPAGES_PROJECT && -f /etc/sysconfig/epages6 ]] ; then
   . /etc/sysconfig/epages6
  VARIABLES_SOURCE=sysconfig
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
fi

if [[ -z $EPAGES_PROJECT ]] ; then
  VARIABLES_SOURCE=default
  EPAGES_PROJECT=epages6
fi

unset COMMON_VER
if rpm --quiet -q epages-common ; then
  EPAGES_PARENT="$(rpm -q epages-common --qf '[%{VERSION} %{INSTPREFIXES}\n]' | head -1)"
  X="${EPAGES_PARENT%%.*}" ; Y="${EPAGES_PARENT%% *}" ; Y=${Y#*.} ; Y=${Y%%.*}
  COMMON_VER=$X.$Y
  EPAGES_PARENT="${EPAGES_PARENT#* }"
fi

EPAGES_INSTALLED=1
if [[ -f /etc/default/$EPAGES_PROJECT ]] ; then
  . /etc/default/$EPAGES_PROJECT
elif [[ -n $COMMON_VER ]] ; then
 EPAGES="$EPAGES_PARENT"/eproot
 EPAGES_SHARED="$EPAGES"/Shared
 EPAGES_CONFIG="$EPAGES_SHARED"/Config
else
  unset EPAGES_INSTALLED
fi

EPAGES_RPMS_URL=ftp://epages-software.de/pub/products/RPMS

unset OWN_REPO DEV_REPO
if [[ $1 = -repo && -n $2 ]] ; then
  EPAGES_RPMS_URL=$2
  OWN_REPO=" $1 $2"
  case $EPAGES_RPMS_URL in
    http://ep-server/yum/6.*.[0-9]/6.*.[0-9])
      DEV_REPO=${EPAGES_RPMS_URL##*/} ; EPAGES_RPMS_URL=${EPAGES_RPMS_URL%/*}
      NEW_VER=${EPAGES_RPMS_URL##*/} ;;
    http://ep-server/yum/6.*.[0-9])
      NEW_VER=${EPAGES_RPMS_URL##*/} ;;
    ftp://ftp.epages.de/pub/upload/6/*/[0-9]/6.*.[0-9])
      DEV_REPO=${EPAGES_RPMS_URL##*/} ; EPAGES_RPMS_URL=${EPAGES_RPMS_URL%/*}
      NEW_VER=${EPAGES_RPMS_URL#ftp://ftp.epages.de/pub/upload/} ; NEW_VER=${NEW_VER//\//.};;
    ftp://ftp.epages.de/pub/upload/6/*/[0-9])
      NEW_VER=${EPAGES_RPMS_URL#ftp://ftp.epages.de/pub/upload/} ; NEW_VER=${NEW_VER//\//.};;
    */RPMS/*) true ;;
    *) echo "Error: $0$OWN_REPO: repository not supported" ; exit 1 ;;
  esac
  shift 2
fi

# -----------------------------------------------------------------------------
# find current version
# -----------------------------------------------------------------------------

unset CURRENT_VER
if [[ -n $EPAGES_INSTALLED ]] ; then

  if grep -q '^\[Cartridges\]' $EPAGES_CONFIG/Database.d/*.conf >/dev/null 2>&1 ; then
    CONF=$EPAGES_CONFIG/Database.d/\*.conf
    CONFFILE=$EPAGES_CONFIG/Database.d/*.conf
  elif ls $EPAGES_CONFIG/Cartridges.d/*.conf >/dev/null 2>&1 ; then
    CONF=$EPAGES_CONFIG/Cartridges.d/\*.conf
    CONFFILE=$EPAGES_CONFIG/Cartridges.d/*.conf
  elif [[ -f $EPAGES_CONFIG/Cartridges.conf ]] ; then
    CONF=$EPAGES_CONFIG/Cartridges.conf
    CONFFILE=$EPAGES_CONFIG/Cartridges.conf
  else
    echo "Error: Install failed, at least one of following files must exist:"
    echo "  \$EPAGES_CONFIG/Database.d/*.conf"
    echo "  \$EPAGES_CONFIG/Cartridges.d/*.conf"
    echo "  \$EPAGES_CONFIG/Cartridges.conf"
    exit 1
  fi

  CURRENT_VER=$(awk -F= '/^DE_EPAGES::Core=/{print $2;exit 0}' $CONFFILE)
  if [[ -z $CURRENT_VER ]] ; then
    echo "Error: Install failed: Can't find DE_EPAGES::Core= in '$CONFFILE'."
    exit 1
  fi
fi

while IFS=. read -a i ; do REL=(${i[@]}) ; done <<<${CURRENT_VER:-$NEW_VER}

[[ -n $EPAGES_INSTALLED ]] && MSG="$CONF: DE_EPAGES::Core=" || MSG="repository $EPAGES_RPMS_URL: "

if [[ ${REL[0]} -ne 6 || ${REL[1]} -lt 9 || -z ${REL[2]} ]] ; then
  echo "Error: Install failed: $MSG${CURRENT_VER:-$NEW_VER}: void version"
  exit 1
fi

X=${REL[0]} ; Y=${REL[1]} ; Z=${REL[2]}
UPDATE_VER=$X.$Y

# -----------------------------------------------------------------------------
# check kernel version
# -----------------------------------------------------------------------------

[[ "`uname -p`" = x86_64 ]] && ARCH=x86_64 || ARCH=i386

case `uname -r` in
  2.6.16[.-]*) KERNEL=2.6.16s ;;
  2.6.18[.-]*) KERNEL=2.6.18r ;;
  2.6.27[.-]*) KERNEL=2.6.27s ;;
  2.6.32-*)    KERNEL=2.6.32r ;;
  2.6.32.*)    KERNEL=2.6.32s ;;
  *) echo "Error: Install failed: `uname -r`: Linux kernel version not supported."
     exit 1 ;;
esac

# -----------------------------------------------------------------------------
# check database type
# -----------------------------------------------------------------------------

cat $EPAGES_CONFIG/Database.conf $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null \
| grep -q '^[[:space:]]*datasource=dbi:Sybase' && SYBASE_CLIENT=1 || unset SYBASE_CLIENT

cat $EPAGES_CONFIG/Database.conf $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null \
| grep -q '^[[:space:]]*datasource=dbi:mysql' && MYSQL_CLIENT=1 || unset MYSQL_CLIENT

unset DB_CLIENT_RPMS
if [[ ( -x $EPSERVICE && -n $SYBASE_CLIENT ) || -r $RUNDB$SERVER ]] ; then
  DB_CLIENT_RPMS="sybcmn sybcli"
fi
if [[ ( -x $EPSERVICE && -n $MYSQL_CLIENT ) || -d /var/lib/mysql/sitedb || -r $EPAGES/Setup/SetupMySQL.sh ]] ; then
  DB_CLIENT_RPMS="$DB_CLIENT_RPMS mysql"
fi

# -----------------------------------------------------------------------------
# check installed servers
# -----------------------------------------------------------------------------

unset SERVER_RPMS

ls /etc/{apache2,httpd}/{vhosts.d,conf.d}/zzz-epages-httpd.conf 2>/dev/null \
| grep -q  . && SERVER_RPMS="webconf"

[[ -x $EPSERVICE ]] && SERVER_RPMS="$SERVER_RPMS appsrv"
[[ -x $EP_RR ]] && SERVER_RPMS="$SERVER_RPMS reqsrv"
[[ -x $HTTPD__SV ]] && SERVER_RPMS="$SERVER_RPMS websrv"
[[ -r $RUNDB$SERVER ]] && SERVER_RPMS="$SERVER_RPMS sybsrv"
[[ -d /var/lib/mysql/sitedb || -r $EPAGES/Setup/SetupMySQL.sh ]] && SERVER_RPMS="$SERVER_RPMS mysqld"

# -----------------------------------------------------------------------------
# create symlinks /srv/epages/* if necessary
# -----------------------------------------------------------------------------

EPAGES_PARENT=/srv/epages
if [[ -n $EPAGES_INSTALLED ]] ; then
  [[ -d $EPAGES ]] && EPAGES_PARENT=$(cd $EPAGES/.. ; pwd) || EPAGES_PARENT=$(dirname $EPAGES)
fi

if [[ -n $EPAGES_INSTALLED && $(find $EPAGES_SHARED -maxdepth 0 -printf "%i") != $(find /srv/epages/eproot/Shared -maxdepth 0 -printf "%i" 2>/dev/null) ]] ; then
  mkdir -m 755 -p /srv/epages
  for i in etc docs eproot ; do
    [ -d $EPAGES_PARENT/$i ] || continue
    rm -f /srv/epages/$i 2>/dev/null
    [ -d /srv/epages/$i ] && mv /srv/epages/$i /srv/epages/$i~$(date +%s)~
    ln -s $EPAGES_PARENT/$i /srv/epages/$i
  done
fi

# -----------------------------------------------------------------------------
# find RPMS to install
# -----------------------------------------------------------------------------

echo "Using repository: $EPAGES_RPMS_URL"
echo "Using new version: $NEW_VER"
echo "Using install prefix: $EPAGES_PARENT"
echo "Using project name: $EPAGES_PROJECT"

[[ $VARIABLES_SOURCE = sysconfig ]] && echo "  Using sysconfig variables: /etc/sysconfig/epages6"
[[ -f /etc/default/$EPAGES_PROJECT ]] && echo "Using default variables: /etc/default/$EPAGES_PROJECT"

REPO_VER=${COMMON_VER:-$UPDATE_VER}
DEV_VER=${DEV_REPO:-$REPO_VER}

USED_VERSION=$REPO_VER
[[ $UPDATE_VER != $REPO_VER ]] && USED_VERSION="$USED_VERSION/$UPDATE_VER"
[[ -n $DEV_REPO && $DEV_REPO != $REPO_VER ]] && USED_VERSION="$USED_VERSION/$DEV_REPO"
echo "Using repo version: $USED_VERSION"

unset REQ RELEASE_RPM INSTALL_RPM SUSE_REQ INSTALLPREFIX
PATCH_VER=$(rpm -q epages-patch --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$' | head -1)

# for suse add all requirements unless epages is installed
if [[ -z $EPAGES_INSTALLED && ${KERNEL%s} != $KERNEL ]] ; then
  DB_CLIENT_RPMS=mysql
  SERVER_RPMS="appsrv reqsrv mysqld webconf"
fi

for i in release requires common patch_${X}_${Y}_${Z} patch perl $DB_CLIENT_RPMS $SERVER_RPMS ; do

  [[ $i = patch_${X}_${Y}_${Z} && ${X}_${Y}_${Z} = 6_10_0 ]] && continue

  if rpm --quiet -q epages-$i ; then
    DIFFERENT_PLACE=$(rpm -q epages-$i --qf '[%{INSTPREFIXES}\n]' | egrep -v "^($EPAGES_PARENT|[^/]+)$")
    if [[ -n $DIFFERENT_PLACE ]] ; then
      echo "Error: Can't use $BN, ePages is already installed in a different place:"
      echo "  rpm -q epages-$i --qf '%{INSTALLPREFIX}' == $DIFFERENT_PLACE"
      exit 1
    fi
    RPM_VER=$(rpm -q epages-$i --qf '[%{VERSION}\n]' | head -1)

    if [[ ${KERNEL%s} != $KERNEL && $PATCH_VER != ${NEW_VER:-$CURRENT_VER} && $RPM_VER != ${NEW_VER:-$CURRENT_VER} ]] ; then
      case $i in
        release|patch*) true ;;
        requires) SUSE_REQ="$SUSE_REQ $EPAGES_RPMS_URL/kernel/$KERNEL/epages-$i-$KERNEL-0.i386.rpm" ;;
        webconf) SUSE_REQ="$SUSE_REQ $EPAGES_RPMS_URL/rpmify/kernel/epages-$i-$KERNEL-0.i386.rpm" ;;
        sybcmn|sybsrv) SUSE_REQ="$SUSE_REQ $EPAGES_RPMS_URL/rpmify/epages-$i/epages-$i-${NEW_VER:-$CURRENT_VER}-0.$ARCH.rpm" ;;
        *) SUSE_REQ="$SUSE_REQ $EPAGES_RPMS_URL/rpmify/epages-$i/epages-$i-${NEW_VER:-$CURRENT_VER}-0.i386.rpm" ;;
      esac
    fi
  else
    case $i in
      release) RELEASE_RPM="$EPAGES_RPMS_URL/epages-release.noarch.rpm" ;;
      requires) REQ="$REQ $EPAGES_RPMS_URL/kernel/$KERNEL/epages-$i-$KERNEL-0.i386.rpm" ;;
      webconf) REQ="$REQ $EPAGES_RPMS_URL/rpmify/kernel/epages-$i-$KERNEL-0.i386.rpm" ;;
      patch_${X}_${Y}_${Z}) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-patch/epages-$i-${CURRENT_VER:-$NEW_VER}-0.i386.rpm" ;;
      sybcmn|sybsrv) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-$i/epages-$i-${CURRENT_VER:-$NEW_VER}-0.$ARCH.rpm" ;;
      *) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-$i/epages-$i-${CURRENT_VER:-$NEW_VER}-0.i386.rpm" ;;
    esac
  fi
done

[[ -z $EPAGES_INSTALLED && ${KERNEL%s} != $KERNEL ]] && SUSE_REQ="$SUSE_REQ $REQ"
[[ -z $EPAGES_INSTALLED ]] && unset REQ

# -----------------------------------------------------------------------------
# install RPMs
# -----------------------------------------------------------------------------

export LANG=C

if [[ -z $RELEASE_RPM ]] ; then
  for i in epages.repo epages-kernel.repo epages-setup.repo ; do
    [[ ! -f /etc/yum.repos.d/$i ]] && RELEASE_RPM="$EPAGES_RPMS_URL/epages-release.noarch.rpm"
  done
  [[ -n $RELEASE_RPM ]] && rpm -e epages-release
fi

[[ -n $RELEASE_RPM ]] && rpm -hiv $@ $RELEASE_RPM 2>&1 | tee /tmp/${BN%.sh}.~$$~
rpm --quiet -q epages-release && RELEASE_INSTALLED=1 || unset RELEASE_INSTALLED

if [[ -n $RELEASE_INSTALLED && ! -f /etc/yum.repos.d/epages-update.repo ]] ; then
  sed 's/setup/update/' /etc/yum.repos.d/epages-setup.repo > /etc/yum.repos.d/epages-update.repo
  sed 's/setup/update/' /etc/yum.repos.d/mirrors-epages-setup > /etc/yum.repos.d/mirrors-epages-update
  chmod 644 /etc/yum.repos.d/*epages-update*
fi

if [[ -n $RELEASE_INSTALLED ]] ; then
  EDIT_CNF=/var/epages/ini-config.sh
  [[ ! -e $EDIT_CNF ]] && EDIT_CNF=/var/tmp/ini-config.sh
  [[ ! -e $EDIT_CNF ]] && EDIT_CNF="$EPAGES"/bin/ini-config.sh
  [[ ! -e $EDIT_CNF ]] && EDIT_CNF=/var/epages/ini-config.sh
  $EDIT_CNF -f /etc/yum.repos.d/epages-update.repo -s epages-update -k enabled -v 1 -i || exit 1
fi

if [[ -n $RELEASE_INSTALLED && -n $CURRENT_VER ]] ; then
  for i in /etc/yum.repos.d/*epages-update* ; do
    sed -i -r "s,/[0-9]+\.[0-9]+-update,/$UPDATE_VER-update," $i
  done
fi

if [[ -n $SUSE_REQ ]] ; then
  eval rpm -hiv $@ --test --prefix $EPAGES_PARENT $SUSE_REQ >> /tmp/${BN%.sh}.~$$~ 2>&1
fi

if [[ -n $REQ ]] ; then
  eval rpm -hiv $@ --prefix $EPAGES_PARENT $REQ 2>&1 | tee -a /tmp/${BN%.sh}.~$$~
fi

# -----------------------------------------------------------------------------
# find missing capabilities
# -----------------------------------------------------------------------------

unset REQ

# some capabilities needed for epages packages are missing
for cap in $(awk '/ is needed /{print $1}' /tmp/${BN%.sh}.~$$~ 2>/dev/null) ; do

  case $cap in
    apache2|apache2-worker|audit-libs|bind-utils|coreutils) REQ="$REQ $cap" ;;
    gawk|httpd|libstdc++|make|mod_ssl|net-tools|pam|sed|file|gd) REQ="$REQ $cap" ;;
    procps|python-urlgrabber|rpm-python|java-1.6.0-openjdk) REQ="$REQ $cap" ;;
    dejavu-*-fonts) REQ="$REQ $cap" ;;
    jre-openjdk) REQ="$REQ java-1.6.0-openjdk" ;;
    jre-sun) REQ="$REQ java-1_4_2-sun" ;;
    jre-ibm) REQ="$REQ java-1_6_0-ibm" ;;
  esac

  K="$KERNEL-$ARCH"
  case $K in
    2.6.32r-x86_64) I86=.i686 ;;
    2.6.18r-x86_64) I86=.i386 ;;
    *s-*) I86=.i586 ;;
    *) unset I86 ;;
  esac

  case $K in
    *r-*)
    X11=$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) REQ="$REQ alsa-lib$I86" ;;
      /lib/libgcc_s.so.1) REQ="$REQ libgcc$I86" ;;
      /lib64/libgcc_s.so.1) REQ="$REQ libgcc" ;;
      /lib/libpam.so.0) REQ="$REQ pam$I86" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) REQ="$REQ audit-libs$I86" ;;
      /lib64/libaudit.so.0) REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33$I86" ;;
      /usr/lib64/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33" ;;
      /usr/lib/libstdc++.so.6) REQ="$REQ libstdc++$I86" ;;
      /usr/lib64/libstdc++.so.6) REQ="$REQ libstdc++" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) REQ="$REQ compat-libstdc++-296" ;;
      /sbin/chkconfig) REQ="$REQ chkconfig" ;;
      /usr/bin/crontab) [ $K = ${K#2.6.18} ] && REQ="$REQ cronie" || REQ="$REQ vixie-cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc-common" ;;
      /usr/bin/mysql) REQ="$REQ mysql" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql-server" ;;
      /usr/lib/libjpeg.so.62) REQ="$REQ libjpeg$I86" ;;
      /usr/lib/libpng12.so.0) REQ="$REQ libpng$I86" ;;
      /usr/sbin/sendmail) REQ="$REQ sendmail" ;;
      libgssapi_krb5.so.2) REQ="$REQ krb5-libs$I86" ;;
      */libz.so.0) REQ="$REQ zlib$I86" ;;
      /lib/libexpat.so.*) REQ="$REQ expat$I86" ;;
      /usr/lib/libexpat.so.0) REQ="$REQ compat-expat1$I86" ;;
      /usr/lib/libgdbm.so.2) REQ="$REQ gdbm$I86" ;;
      gnupg) REQ="$REQ gnupg2" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) REQ="$REQ libxslt$I86" ;;
      /usr/lib/libfreetype.so.6) REQ="$REQ freetype$I86" ;;
      /usr/lib/libfontconfig.so.1) REQ="$REQ fontconfig$I86" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) REQ="$REQ $X11$I86" ;;
      /usr/lib/libssl.so.6) [ $K = ${K#2.6.32} ] && REQ="$REQ openssl$I86" || REQ="$REQ openssl098e$I86" ;;
      /usr/lib/libssl.so.10|/lib/libssl.so.6) REQ="$REQ openssl$I86" ;;
      /usr/lib/libgd.so.2) REQ="$REQ gd$I86" ;;
      /lib/libuuid.so.1) [ $K = ${K#2.6.18} ] && REQ="$REQ libuuid$I86" || REQ="$REQ e2fsprogs-libs$I86" ;;
      /usr/lib/libltdl.so.7) REQ="$REQ libtool-ltdl$I86" ;;
    esac ;;
    *s-*)
    [[ $K != ${K#2.6.16} || $cap = libXi.so.6 || $cap = libXtst.so.6 ]] && X11=xorg-x11-libs || X11=xorg-x11-$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) [ $K != ${K#2.6.16} ] && REQ="$REQ alsa" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libasound2 libasound2-32bit" || REQ="$REQ libasound2"; } ;;
      /lib/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libgcc43-32bit" || REQ="$REQ libgcc43"; } ;;
      /lib64/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || REQ="$REQ libgcc43" ;;
      /lib/libpam.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ pam-32bit" || REQ="$REQ pam" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ audit-libs-32bit" || REQ="$REQ audit-libs" ;;
      /lib64/libaudit.so.0) REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++33-32bit" || REQ="$REQ libstdc++33"; } ;;
      /usr/lib64/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || REQ="$REQ libstdc++33" ;;
      /usr/lib/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++43-32bit" || REQ="$REQ libstdc++43"; } ;;
      /usr/lib64/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || REQ="$REQ libstdc++43" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) [ $K != ${K%x86_64} ] && REQ="$REQ compat-32bit"  || REQ="$REQ compat" ;;
      /sbin/chkconfig) REQ="$REQ aaa_base" ;;
      /usr/bin/crontab) REQ="$REQ cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc" ;;
      /usr/bin/mysql) REQ="$REQ mysql-client" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql" ;;
      /usr/lib/libjpeg.so.62) [ $K != ${K%x86_64} ] && REQ="$REQ libjpeg-32bit" || REQ="$REQ libjpeg" ;;
      /usr/lib/libpng12.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ libpng-32bit" || REQ="$REQ libpng" ;;
      /usr/sbin/sendmail) REQ="$REQ postfix" ;;
      libgssapi_krb5.so.2) [ $K != ${K%x86_64} ] && REQ="$REQ krb5-32bit" || REQ="$REQ krb5" ;;
      */libz.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ zlib-32bit"  || REQ="$REQ zlib" ;;
      /lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ libexpat1-32bit"  || REQ="$REQ libexpat1" ;;
      /usr/lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ expat-32bit"  || REQ="$REQ expat" ;;
      gnupg) [ $K = ${K#2.6.16} ] && REQ="$REQ gpg2" || REQ="$REQ gpg" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ libxslt-32bit" || REQ="$REQ libxslt" ;;
      /usr/lib/libfreetype.so.6) [ $K != ${K%x86_64} ] && REQ="$REQ freetype2-32bit" || REQ="$REQ freetype2" ;;
      /usr/lib/libfontconfig.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ fontconfig-32bit" || REQ="$REQ fontconfig" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) [ $K != ${K%x86_64} ] && REQ="$REQ $X11-32bit" || REQ="$REQ $X11" ;;
      /usr/lib/libssl.so.0.9.8) [ $K != ${K#2.6.16} ] && {
        [ $K != ${K%x86_64} ] && REQ="$REQ openssl-32bit" || REQ="$REQ openssl"; } || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libopenssl0_9_8-32bit" || REQ="$REQ libopenssl0_9_8"; } ;;
      /lib/libuuid.so.1) [ $K != ${K#2.6.16} ] && {
        [ $K != ${K%x86_64} ] && REQ="$REQ e2fsprogs-32bit" || REQ="$REQ e2fsprogs"; } || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libuuid1-32bit" || REQ="$REQ libuuid1"; } ;;
      /usr/lib/libltdl.so.7) [ $K != ${K%x86_64} ] && REQ="$REQ libltdl7-32bit" || REQ="$REQ libltdl7" ;;
    esac ;;
  esac

done

if [[ -n "$REQ" ]] ; then
  REQ=$(echo $REQ | tr ' ' '\n' | sort | uniq | tr '\n' ' ')
  rpm -e epages-release 2>/dev/null
  rm -f /etc/yum.repos.d/*epages-{setup,update}*
  echo
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo "First install missing RPM packages by executing:"
  [ $KERNEL = ${KERNEL%s} ] && INSTALLER=yum || INSTALLER=zypper
  echo "$INSTALLER install $REQ"
  echo "Then re-run:"
  echo "$0$OWN_REPO $@"
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo
  RET=1
else
  if [[ -n $RELEASE_INSTALLED && ( -n $OWN_REPO || -z $RELEASE_RPM ) ]] && cd /etc/yum.repos.d ; then
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/$DEV_VER/\$basearch," epages.repo
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/$REPO_VER-setup," epages-setup.repo
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/$UPDATE_VER-update," epages-update.repo
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/kernel/$KERNEL," epages-kernel.repo

    echo "$EPAGES_RPMS_URL/$DEV_VER/\$ARCH" > mirrors-epages
    echo "$EPAGES_RPMS_URL/$REPO_VER-setup" > mirrors-epages-setup
    echo "$EPAGES_RPMS_URL/$UPDATE_VER-update" > mirrors-epages-update
    echo "$EPAGES_RPMS_URL/kernel/$KERNEL" > mirrors-epages-kernel-$KERNEL

    if [[ -n $DEV_REPO ]] ; then
      sed -i -r "s,^(\s*enabled\b).*,\1 = 0," epages-setup.repo
      sed -i -r "s,^(\s*enabled\b).*,\1 = 0," epages-update.repo
    fi
  fi
  grep -q 'error:' /tmp/${BN%.sh}.~$$~ 2>/dev/null && RET=1 || RET=0
fi

exit $RET
