#!/bin/env perl
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBN8drul5jMabHSgyARAuTSAKDVXGZoI3du79GnLilcaqsRbaG7NwCeMBEc
#JVs96USF1YZ5+flSFAMhRzs=
#=sWUu
#-----END PGP SIGNATURE-----
#line 2

use strict;
use English;

sub usage {
    print <<END_USAGE;
Description:
    This script replaces <LOOP:PARAM ...PARAM.../> by ...VALUE...
    in file Input for each value as defined on command line

Usage:
    [LOOP_SEP="SEP"] perl $0 [-PARAM "VALUES"] < Input > Output

Options:
    -PARAM "VALUES" : use your variable name instead of PARAM
                      use your loop values instead of VALUES
    LOOP_SEP="SEP"  : envvar LOOP_SEP defines separator in VALUES (default: \\s+)

Example:
    Input:   <LOOP:xxx member:xxx />
    Command: perl writeLoop.PL -xxx "peter paul mary" < Input > Output
    Output:  member:peter member:paul member:mary
    Command: LOOP_SEP=, perl writeLoop.PL -xxx "peter,paul,mary" < Input > Output
    Output:  member:peter member:paul member:mary
END_USAGE
;
    exit 2;
}

local $| = 1;

my %Options;
my ($Param);

my $Sep = $ENV{LOOP_SEP} ? $ENV{LOOP_SEP} : '\s+';

usage() unless scalar @ARGV;

while (my $Arg = shift @ARGV) {
    if ($Arg =~ /^-+(.*)/) {
        usage() if $Param;
        $Param = $1;
    } else {
        usage() unless $Param;
        $Options{$Param} = $Arg;
        $Param = '';
    }
}
usage() if $Param;

local $/ = undef;
my $Content = <>;

while ($Content =~ /<LOOP:(\S+)\s(.+?)\/>/s) {
    my $PreMatch = $PREMATCH; ##no critic(ProhibitMatchVars)
    my $PostMatch = $POSTMATCH; ##no critic(ProhibitMatchVars)
    my $Parameter = $1;
    my $Template = $2;
    my $Match = $Options{$Parameter} ? '' : $2;
    foreach my $Member (split(/$Sep/,$Options{$Parameter})) {
        my $Loop = $Template;
        $Loop =~ s/\Q$Parameter\E/$Member/gs;
        $Match .= $Loop;
    }
    $Content = "$PreMatch$Match$PostMatch";
}
print $Content;

1;
